/*
    Conventions
    
    1.Processing phases
        - ASP processing:
            Processing an ASP page to plain VBScript
        - Pass A:
            (Analysis) Gathering information
            Processing structure - includes building of the symbol tables
        - Pass B:
            (Generation)

*/


// Error constants

// Dim statements
var cERR_DIMSyntax              = 201;

// Arglists
var cERR_SetLetRwuiredArgs      = 301;

// Definitions
var cERR_PropertySyntax         = 401;
var cERR_ConstructorArgs        = 402;
var cERR_ConstructorAccess      = 403;
var cERR_DestructorArgs         = 404;
var cERR_DestructorAccess       = 405;
var cERR_ArgListSyntax          = 406;
var cERR_DuplicateNames         = 407;

// Statments
var cErr_SyntaxError            = 501;
var cErr_UnexpectedStatement    = 502;
var cWarn_UnexpectedDirective   = 503;

// Features
var cErr_NotAllowed             = 601;

// Compile-time scripting
var cErr_CompilerLanguageNotDef   = 701;
var cErr_LanguageNotDef           = 701;
var cErr_EngineLoadFailed         = 702;
var cErr_FailedToAdd              = 703;
var cErr_ScriptCompileErr         = 704;
var cErr_ScriptRunTimeErr         = 705;
var cErr_IDNotDef                 = 706;
var cErr_ProgIDNotDef             = 707;
var cErr_MiscErr                  = 708;
var cErr_RequiredEventNotHandled  = 709;


// Special
var cERR_UnExpected             = 10001;
var cERR_ReservedName           = 10002;
var cERR_UndeclaredSymbol       = 10003;
var cERR_ClassnotFound          = 10004;
var cERR_Unaccessible           = 10005;
var cERR_Internal               = 10006;
var cERR_FileNotFound           = 10007;

var cDefaultContextEntryName    = "ASPCParent";
var cDefaultMainProcName        = "ASPCExecuteClass";


// File types
var cFileInput          =   1,
    cFileOutput         =   2,
    cFileVB             =   3,
    cFileHidden         =   0,
    cFileInclude        =   4;




// Global regular expressions
var reEOL = /[\n\r]+/gi;
var reQuotes = /\"/gi;
var reFolder = /\\/gi;
var reVirt = /\//gi;
var reDot = /\./gi;
var reTrimL = /^\s+/gi;
var reTrimR = /\s+$/gi;
var reClassName = /\w{2,30}/gi;

// Misc routines
function CR(x) {
    if (x == 0) {
        g_compilationResult = 0;
        return;
    }
    if (x == -1) {
        if (g_compilationResult != 0) g_compilationResult = -1;
        return;
    }
}


function EscapeQuotes(str) {
    return str.replace(reQuotes,"\"\"");
}
function WriteHTML(f,str) {
    var arr = str.split(reEOL);
    var i;
    if (arr != null) {
        for (i = 0; i < arr.length; i++) {
            if (arr[i].length != 0) {
                if (i < (arr.length - 1)) {
                    f.WriteText(g_ResponseWrite + " \"" + EscapeQuotes(arr[i]) + "\" & vbCrLf",1);
                } else {
                    f.WriteText(g_ResponseWrite + " \"" + EscapeQuotes(arr[i]) + "\"",1);
                }
            }
        }
    }
}

// Special routines
function CreateDirectory(str) {
    var s = new String(str);
    var arr = s.split(reFolder);
    if (arr == null) return false;
    var i;
    var strCurrent = arr[0];
    if (!fso.FolderExists(strCurrent)) {
        compiler.ReportError(8,"Root of the specified folder does not exist: " + strCurrent,0);
        return false;
    }
    for (i = 1; i < arr.length; i++) {
        strCurrent += "\\" + arr[i];
        if (!fso.FolderExists(strCurrent)) {
            // Create it
            compiler.ReportMessage(0,"Creating folder: " + strCurrent);
            fso.CreateDirectory(strCurrent);
        }
    }
    return true;
}
function Trim(str) {
    var s = new String(str);
    s = s.replace(reTrimL,"");
    s = s.replace(reTrimR,"");
    return s;
}
function CombinePaths(p1,p2) {
    var r = new String(p1);
    var r1 = new String(p2);
    if (r[r.length-1] != "\\") r += "\\";
    if (r1[0] == "\\") r1 = r1.slice(1,ri.length - 1);
    r += r1;
    return r;
}
function GetPathOf(str) {
    var r = new String(str);
    if (r[r.length-1] == "\\") return r;
    var arr = r.split(reFolder);
    if (arr == null) return null;
    if (arr.length == 1) return "";
    r = arr[0];
    var i;
    for (i = 1; i < (arr.length - 1); i++) {
        r += "\\" + arr[i];
    }
    return r;
}

function GetRelPath(rootIN,str) {
    var r = new String(str);
    var root = new String(rootIN);
    var i;
    i = r.indexOf(root);
    
    if (i == 0) {
       r = r.slice(root.length);
       
       if (r.charAt(0) == "\\") {
            r = r.slice(1);
       }
       return r;
    } else {
        return null;
    }
}
function CreatePathsForFiles(root,sect) {
    var i;
    var str;
    var boolResult = true;
    compiler.ReportMessage(0,"Preparing paths for the output ASP files");
    for (i = 1; i <= sect.Count; i++) {
        compiler.ReportMessage(0,"File: " + sect(i)("File"));
        str = GetRelPath(root,sect(i)("File"));
        if (str == null) {
            boolResult = false;
            CR(0);
            compiler.ReportFile(cFileInput,sect(i)("File"),"Input file");
            compiler.ReportError(1,"File is not under the project site root: " + root,0);
            compiler.ReportFile(cFileHidden,"","");
        } else {
            CreateDirectory(GetPathOf(CombinePaths(siteRootPathOutput,str)));
        }
    }
    return boolResult;
}
function CreateClassModuleName(strInput) {
    var arr = strInput.split(reFolder);
    var r = "cls" + progIDNum + arr[arr.length - 1];
    var r = r.replace(reDot,"_");
    progIDNum ++;
    return r;
}
/* fileSpec - the specified path to the compile time script
 * curFile - full path
 */
function GetCompileTimeScriptPath(fileSpec,curFile) {
    var f = new String(fileSpec);
    var fileName,i,rec;
    f = f.replace(reVirt,"\\");
    if (f.charAt(0) == "\\") {
        if (typeof(cfg("CompileTimeScripting")("IncludeDir")) == "object") {
            // Try them
            rec = cfg("CompileTimeScripting")("IncludeDir");
            for (i = 1; i <= rec.Count; i++) {
                fileName = rec(i) + f;
                if (fso.FileExists(fileName)) {
                    return fileName;
                }
            }
        }
        // Not found in globals - try the site
        fileName = siteRootPath + f;
        if (fso.FileExists(fileName)) {
            return fileName;
        }
    } else {
        // relative path
        fileName = CombinePaths(GetPathOf(curFile),f);
        if (fso.FileExists(fileName)) {
            return fileName;
        }
    }
    return null;
}
function LoadScriptFromFile(fileSpec,curFile) {
    var fileName = GetCompileTimeScriptPath(fileSpec,curFile);
    if (fileName == null) return null;
    var file,str;
    file = fso.OpenFile(fileName,0x10);
    str = file.ReadText();
    file.Close();
    return str;
}

/////////////////////////////////////////// ASP/SSI PARSING ///////////////////////////////////////////////
var g_nSegment;
function CleanUpEngines(col) {
    var e = new Enumerator(col);
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        o.Stop();
        o.Close();
        o.Clear();
    }
}   

function ConstructFile(strFile,parserToUse) {
    compiler.ReportTrace(0,"ConstructFile(" + strFile + ")");
    var c = new FileConstruct(strFile,parserToUse);
    if (!c.Result) return null;
    return c.tree;
}
function FileConstruct(strFile,parserToUse) {
    compiler.ReportTrace(0,"FileConstruct(" + strFile + ")");
    this.tree = new ActiveXObject("newobjects.utilctls.VarDictionary");
    this.tree.enumItems = true;
    this.tree.allowUnnamedValues = true;
    this.tree.allowDuplicateNames = true;
    this.Result = true;
    var i;
    g_nSegment = 0;
    
    if (!ConstructPutFile(this.tree,strFile,parserToUse)) {
        this.Result = false;
    } else {
        this.colScripts = new ActiveXObject("newobjects.utilctls.VarDictionary");
        this.colScripts.enumItems = true;
        this.colScripts.allowUnnamedValues = true;
        this.colScripts.allowDuplicateNames = true;
        
        this.colScriptDisps = new ActiveXObject("newobjects.utilctls.VarDictionary");
        this.colScriptDisps.enumItems = true;
        this.colScriptDisps.allowUnnamedValues = true;
        this.colScriptDisps.allowDuplicateNames = true;
        
        this.colCostomTags = new ActiveXObject("newobjects.utilctls.VarDictionary");
        this.colCostomTags.enumItems = true;
        this.colCostomTags.allowUnnamedValues = true;
        this.colCostomTags.allowDuplicateNames = true;
        
        this.curFile = strFile;
        this.mainFile = strFile;
        this.tagNum = 0;
        
        if (!this.EnumTreeDoCTS(this.tree)) {
            // Failure
            this.Result = false;
        } else {
            // Success
            // Script load successful
            // Run all the scripts
            for (i = 1; i <= this.colScripts.Count; i++) {
                if (!this.colScripts(i).Run()) {
                    /////////////////////////////////////////
                    compiler.ReportFile(cFileHidden,this.colScripts(i)("Info")("CurrentFile"),"Error in COMPILER tag in this file")
                    compiler.ReportError(cErr_ScriptRunTimeErr,"Script run-time error in the COMPILER tag '" + this.colScripts.Key(i) + "' Language=" + this.colScripts(i)("Info")("ScriptLanguage") + ". Error is: " + this.colScripts(i).lastError,0);
                    CR(0);
                    this.colScriptDisps.Clear();
                    this.colCostomTags.Clear();
                    CleanUpEngines(this.colScripts);
                    this.tree.Clear();
                    this.Result = false;
                    return;
                }
                this.colScriptDisps.Add(this.colScripts.Key(i),this.colScripts(i).script);
            }
            
            // Now check if there is something global to call
            var boolResult = true;
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnCheckStructure")) {
                    // Invoke it to perform custom tasks
                    compiler.ReportMessage(0,"Invoking the OnCheckStructure of the global compile-time script.");
                    boolResult = procFileScript.script.OnCheckStructure(this.tree,this.colScriptDisps,this.colCostomTags);
                }
            }
            // Clean up the scripts
            this.colScriptDisps.Clear();
            this.colCostomTags.Clear();
            CleanUpEngines(this.colScripts);
            if (!boolResult) {
                CR(0);
                compiler.ReportError(cErr_MiscErr,"Compile-time script returned errors",0);    
                this.Result = false;
            }
            this.Result = true;                
        }
    }
}
FileConstruct.prototype.EnumTreeDoCTS = function(tree) {
    compiler.ReportTrace(0,"EnumTreeDoCTS(...)");
    var temp,o,tagID,info;
    var str;
    var scr;
        
    var e = new Enumerator(tree);
    
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        if (typeof(o) != "object") continue;
        this.tagNum ++;
        
        if (typeof(o("ID")) != "undefined") {
            tagID = o("ID");
        } else {
            tagID = "tag#" + this.tagNum;
        }
        
        switch (o("Class")) {
            case "CompileTimeScript":
                scr = new ActiveXObject("newObjects.Scphost.ScpMan2");
                if (typeof(o("Language")) == "undefined") {
                    compiler.ReportFile(cFileHidden,this.curFile,"Error in SCRIPT RUNAT=COMPILER tag in this file")
                    compiler.ReportError(cErr_LanguageNotDef,"Language attribute not found in a SCRIPT tag '" + tagID + "'",0);
                    CR(0);
                    this.colScriptDisps.Clear();
                    this.colCostomTags.Clear();
                    CleanUpEngines(this.colScripts);
                    this.tree.Clear();
                    return false;
                }
                // Try to create script engine
                if (!scr.LoadEngine(o("Language"))) {
                    compiler.ReportFile(cFileHidden,this.curFile,"Error in SCRIPT RUNAT=COMPILER tag in this file")
                    compiler.ReportError(cErr_EngineLoadFailed,"Cannot load the specified script engine in the COMPILER tag '" + tagID + "' (" + o("Language") + ")",0);
                    CR(0);
                    this.colScriptDisps.Clear();
                    this.colCostomTags.Clear();
                    CleanUpEngines(this.colScripts);
                    tree.Clear();
                    return false;
                }
                // Load standard namespaces
                scr.Add("Project",cfg);
                scr.Add("Compiler",compiler);
                scr.Add("Node",o);
                scr.Add("ProcessedFile",procFileSection);
                info = new ActiveXObject("newobjects.utilctls.VarDictionary");
                info.Add("CurrentFile",this.curFile);
                info.Add("File",this.mainFile);
                info.Add("FileRelative",procFileName);
                info.Add("ProcType",procFileType);                
                info.Add("SiteRootPath",siteRootPath);
                info.Add("SiteRootPathOutput",siteRootPathOutput);
                info.Add("FilePath",GetPathOf(this.curFile));
                info.Add("FileRelativePath",procFileRelPath);
                info.Add("FileNameOnly",procFileNameOnly);
                info.Add("FileExtension",procFileExt);
                info.Add("ScriptLanguage",o("Language"));
                scr.Add("Info",info);
                
                scr.Add("Tree",this.tree);
                scr.Add("Script",this.colScriptDisps);
                scr.Add("Custom",this.colCostomTags);
                this.colScripts.Add(tagID,scr);
                
                if (typeof(cfg("VARIABLES")) == "object") {
                    scr.Add("GlobalVariables",cfg("VARIABLES"));
                } else {
                    temp = new ActiveXObject("newobjects.utilctls.VarDictionary");
                    scr.Add("GlobalVariables",temp);
                }
                
                if (typeof(procFileSection("VARIABLES")) == "object") {
                    scr.Add("Variables",procFileSection("VARIABLES"));
                } else {
                    temp = new ActiveXObject("newobjects.utilctls.VarDictionary");
                    scr.Add("Variables",temp);
                }
                
                // Try to add the code
                if (typeof(o("src")) != "undefined") {
                    // Load from a file
                    str = LoadScriptFromFile(o("src"),this.curFile);
                    if (str == null) {
                        CR(0);
                        compiler.ReportFile(cFileHidden,this.curFile,"Error in SCRIPT RUNAT=COMPILER tag in this file")
                        compiler.ReportError(cERR_FileNotFound,"Cannot load file specified in SRC attribute of a COMPILER tag '" + tagID + "' SRC=" + o("SRC"),0);    
                        this.colScriptDisps.Clear();
                        this.colCostomTags.Clear();
                        CleanUpEngines(this.colScripts);
                        this.tree.Clear();
                        return false;
                    } 
                    if (!scr.AddText(str)) {
                        compiler.ReportFile(cFileHidden,this.curFile,"Error in SCRIPT RUNAT=COMPILER tag in this file")
                        compiler.ReportError(cErr_ScriptCompileErr,"Script compilation error in a COMPILER tag '" + tagID + "' Language=" + o("Language") + ". Error is: " + scr.lastError,0);
                        CR(0);
                        this.colScriptDisps.Clear();
                        this.colCostomTags.Clear();
                        CleanUpEngines(this.colScripts);
                        this.tree.Clear();
                        return false;
                    }
                    temp = GetCompileTimeScriptPath(o("src"),this.curFile);
                    info.Add("ScriptFile",temp);
                    info.Add("ScriptFilePath",GetPathOf(temp));
                }
                if (o("NOSCRIPTCONTENT") != 1) {
                    if (!scr.AddText(o("Content"))) {
                        compiler.ReportFile(cFileHidden,this.curFile,"Error in SCRIPT RUNAT=COMPILER tag in this file")
                        compiler.ReportError(cErr_ScriptCompileErr,"Script compilation error in a COMPILER tag '" + tagID + "' Language=" + o("Language") + ". Error is: " + scr.lastError,0);
                        CR(0);
                        this.colScriptDisps.Clear();
                        this.colCostomTags.Clear();
                        CleanUpEngines(this.colScripts);
                        this.tree.Clear();
                        return false;
                    }
                }
                // Run it - now it happens after completing the tree
                /*
                if (!scr.Run()) {
                    compiler.ReportFile(cFileHidden,this.curFile,"Error in COMPILER tag in this file")
                    compiler.ReportError(cErr_ScriptRunTimeErr,"Script run-time error in the COMPILER tag '" + tagID + "' Language=" + o("Language") + ". Error is: " + scr.lastError,0);
                    CR(0);
                    this.colScriptDisps.Clear();
                    this.colCostomTags.Clear();
                    CleanUpEngines(this.colScripts);
                    this.tree.Clear();
                    return null;
                }
                this.colScriptDisps.Add(tagID,scr.script);
                */
            break;
            case "Custom":
                this.colCostomTags.Add(tagID,o);
                if (!this.EnumTreeDoCTS(o)) return false;
            break;
            case "IncludeBegin":
                this.curFile = o("Value");
            break;
            case "IncludeEnd":
                this.curFile = o("OuterFile");
            break;
            default:
                if (!this.EnumTreeDoCTS(o)) return false;
        }
    }    
    return true;
}

function ConstructPutFile(tree,strFile,parserToUse) {
    compiler.ReportTrace(0,"ConstructPutFile(" + strFile + ")");
    var inFile;
    if (!fso.FileExists(strFile)) {
        CR(0);
        compiler.ReportError(cERR_FileNotFound,"File not found: " + strFile,0);
        return false;
    }
    inFile  = fso.OpenFile(strFile,0x10);
    compiler.ReportTrace(0,"ConstructPutFile -> Start parsing");
    var r = parserToUse.ParseString(inFile.ReadText());
    compiler.ReportTrace(0,"ConstructPutFile -> End parsing");
    inFile.Close();
    tree.enumItems = true;
    
    return ConstructTranslateTree(tree,r,strFile,parserToUse);
}    

function ConstructTranslateTree(tree,r,strFile,parserToUse) {
    compiler.ReportTrace(0,"ConstructTranslateTree(" + strFile + ")");
    // We have the structure
    // Enum segments and generate a plain file
    var e = new Enumerator(r);
    var e1; 
    var o;
    var str,str1;
    var node,subNode;
    var fileName;
    
    for (; !e.atEnd(); e.moveNext()) {
        //if (typeof(r(e.item())) == "object") {
        if (typeof(e.item()) == "object") {
            compiler.ReportTrace(0,"ConstructTranslateTree segment=" + g_nSegment);
            g_nSegment ++;
            // This is something
            o = e.item();
            switch (o("__class")) {
                case "HTMLEmbedParser.newObjects.Default":
                    // Plain HTML
                    node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                    node.itemsAssignmentAllowed = true;
                    node.allowUnnamedValues = true;
                    node.enumItems = true;
                    tree.Add("__" + g_nSegment,node);
                    node.Add("Class","Text");
                    node.Add("Content",o("__text"));
                break;
                case "HTMLEmbedSearcher.newObjects.ASPProcents":
                    // ASP code
                    switch (o("__content")) {
                        case 3:
                            // Just code
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            tree.Add("__" + g_nSegment,node);
                            node.Add("Class","Script");
                            node.Add("Content",o("Code"));
                        break;
                        case 1:
                            // ASP Assignment (direct output)
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            tree.Add("__" + g_nSegment,node);
                            node.Add("Class","Assignment");
                            node.Add("Content",o("Code"));
                        break;
                        case 2:
                            // Directive - just check it
                            if (typeof(o("Language")) != "undefined") {
                                str = new String(o("Language"));
                                if (str.toUpperCase() != "VBSCRIPT" && str.toUpperCase() != "VBS") {
                                    CR(0);
                                    compiler.ReportError(0,"Unsupported scripting language directive: " + str + " in file " + strFile,0);
                                }
                                node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                                node.itemsAssignmentAllowed = true;
                                node.allowUnnamedValues = true;
                                node.enumItems = true;
                                tree.Add("__" + g_nSegment,node);
                                node.Add("Class","ASPDirective");
                                node.Add("Language",o("Language"));
                            }
                        break;
                    }
                break;
                case "HTMLEmbedSearcher.newObjects.HTMLTagSearch":
                    // ASP code
                    switch (o("__content")) {
                        case 101: // SCRIPT RUNAT=SERVER
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            if (typeof(o("ID")) != "undefined") {
                                if (!tree.Add(o("ID"),node)) {
                                    CR(0);
                                    compiler.ReportError(cErr_FailedToAdd,"Cannot add SCRIPT tag with ID=" + o("ID") + " check if it is not duplicated.",0);
                                }
                            } else {
                                tree.Add("__" + g_nSegment,node);
                            }
                            node.Add("Class","Script");
                            str = new String(o("__text"));
                            str = str.slice(o("__textBegin"),o("__textEnd"));
                            node.Add("Content",str);
                        break;
                        case 102: //Object
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            tree.enumItems = true;
                            if (typeof(o("ID")) != "undefined") {
                                node.Add("ID",o("ID"));
                                if (!tree.Add(o("ID"),node)) {
                                    CR(0);
                                    compiler.ReportError(cErr_FailedToAdd,"Cannot add OBJECT tag with ID=" + o("ID") + " check if it is not duplicated.",0);
                                }
                                if (typeof(o("ProgID")) == "undefined") {
                                    CR(0);
                                    compiler.ReportError(cErr_ProgIDNotDef,"ProgID attribute not found in OBJECT tag with ID=" + o("ID"),0);
                                }
                            } else {
                                CR(0);
                                compiler.ReportError(cErr_IDNotDef,"Cannot add OBJECT without ID attribute",0);
                            }
                            node.Add("Class","Object");
                            node.Add("ProgID",o("ProgID"));
                        break;
                        case 1001: // Compiler
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            if (typeof(o("ID")) != "undefined") {
                                if (!tree.Add(o("ID"),node)) {
                                    CR(0);
                                    compiler.ReportError(cErr_FailedToAdd,"Cannot add COMPILER tag with ID=" + o("ID") + " check if it is not duplicated.",0);
                                }
                            } else {
                                tree.Add("__" + g_nSegment,node);
                            }
                            if (typeof(o("Language")) == "undefined") {
                                CR(0);
                                compiler.ReportError(cErr_LanguageNotDef,"Language attribute not found in SCRIPT RUNAT=Compiler tag",0);
                            }
                            node.Add("Class","CompileTimeScript");
                            str = new String(o("__text"));
                            str = str.slice(o("__textBegin"),o("__textEnd"));
                            node.Add("Content",str);                            
                            // Enum and pass the other attributes
                            
                            for (e1 = 1; e1 <= o.Count; e1++) {
                                switch (o.Key(e1)) {
                                    case "__text":
                                    case "__textBegin":
                                    case "__textEnd":
                                    case "__content":
                                    break;
                                    default:
                                        node.Add(o.Key(e1),o(e1));
                                }
                            }
                        break;
                        case 1002: // CUSTOM
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            
                            if (typeof(o("ID")) != "undefined") {
                                if (!tree.Add(o("ID"),node)) {
                                    CR(0);
                                    compiler.ReportError(cErr_FailedToAdd,"Cannot add CUSTOM tag with ID=" + o("ID") + " check if it is not duplicated.",0);
                                }
                            } else {
                                tree.Add("__" + g_nSegment,node);
                            }
                            node.Add("Class","Custom");
                            str = new String(o("__text"));
                            str = str.slice(o("__textBegin"),o("__textEnd"));
                            node.Add("Content",str);
                            //subNode = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            //subNode.enumItems = true;
                            //subNode.itemsAssignmentAllowed = true;
                            //subNode.allowUnnamedValues = true;
                            //node.Add("Subnodes",subNode);
                            // Enum and pass the other attributes
                            
                            for (e1 = 1; e1 <= o.Count; e1++) {
                                if (typeof(o(e1)) != "object") {
                                    switch (o.Key(e1)) {
                                        case "__text":
                                        case "__textBegin":
                                        case "__textEnd":
                                        case "__content":
                                        break;
                                        default:
                                            node.Add(o.Key(e1),o(e1));
                                    }
                                }
                            }
                            if (!ConstructTranslateTree(node,o,strFile,parserToUse)) return false;
                        break;
                    }
                break;
                case "HTMLEmbedSearcher.newObjects.HTMLComment":
                    switch (o("__content")) {
                        case 4:
                            // Include directive
                            // Recurse
                            
                            if (typeof(o("virtual")) == "string") {
                                fileName = o("virtual");
                                fileName = fileName.replace(reVirt,"\\");
                                fileName = CombinePaths(siteRootPath,fileName);
                                str = "virtual=" + o("virtual");
                                str1 = "virtual";
                            } else {
                                if (typeof(o("file")) == "string") {
                                    fileName = o("file");
                                    fileName = fileName.replace(reVirt,"\\");
                                    fileName = CombinePaths(GetPathOf(strFile),fileName);
                                    str = "file=" + o("file");
                                    str1 = "file";
                                } else {
                                    CR(0);
                                    compiler.ReportError(cERR_Internal,"Internal error in ConstructPutFile",0);
                                    return false;
                                }
                            }
                            compiler.ReportFile(cFileInclude,fileName,"Include file");
                            if (!fso.FileExists(fileName)) {
                                CR(0);
                                compiler.ReportError(cERR_FileNotFound,"Include file not found: " + fileName,0);
                                compiler.ReportMessage(2,"Include directive is: " + str);
                                return false;
                            }
                            // Time to recurse
                            // Mark begin of the include
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            tree.Add("__" + g_nSegment,node);
                            node.Add("Class","IncludeBegin");
                            node.Add("Type",str1);
                            node.Add("Value",o(str1));
                            node.Add("OuterFile",strFile);
                            if (!ConstructPutFile(tree,fileName,parserToUse)) {
                                // Failure - stop processing
                                return false;
                            }
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            tree.Add("__" + g_nSegment,node);
                            node.Add("Class","IncludeEnd");
                            node.Add("Type",str1);
                            node.Add("Value",o(str1));
                            node.Add("OuterFile",strFile);
                        break;
                        case 5:
                            // Just a comment
                            node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                            node.itemsAssignmentAllowed = true;
                            node.allowUnnamedValues = true;
                            node.enumItems = true;
                            tree.Add("__" + g_nSegment,node);
                            node.Add("Class","Comment");
                            node.Add("Content",o("Code"));
                        break;
                    }
                break;
                default:
                    node = new ActiveXObject("newobjects.utilctls.VarDictionary");
                    node.itemsAssignmentAllowed = true;
                    node.allowUnnamedValues = true;
                    node.enumItems = true;
                    
                    if (typeof(o("ID")) != "undefined") {
                        if (!tree.Add(o("ID"),node)) {
                            CR(0);
                            compiler.ReportError(cErr_FailedToAdd,"Cannot add tag with ID=" + o("ID") + " check if it is not duplicated.",0);
                        }
                    } else {
                        tree.Add("__" + g_nSegment,node);
                    }

                    // Enum and pass the other attributes
                    for (e1 = 1; e1 <= o.Count; e1++) {
                        if (typeof(o(e1)) != "object") {
                            node.Add(o.Key(e1),o(e1));
                        }
                    }
                    if (!ConstructTranslateTree(node,o,strFile,parserToUse)) return false;
            }
        }
    }
    return true;
    
}

var _writtenStmtTotal = 0

function WriteTagBegin(o,out) {
    out.WriteText("<" + o("__class"),0);
    var i,str;
    for (i = 1; i <= o.Count; i++) {
        if (typeof(o(i)) == "object") continue;
        str = o.Key(i);
        if (str.slice(0,2) == "__") continue;
        out.WriteText(" " + o.Key(i) + "=\"" + o(i) + "\"",0);
    }
    out.WriteText(">",0);
}
function WriteTagBeginVBS(o,out) {
    var strTag = "";
    strTag += "<" + o("__class");
    var i,str;
    for (i = 1; i <= o.Count; i++) {
        if (typeof(o(i)) == "object") continue;
        str = o.Key(i);
        if (str.slice(0,2) == "__") continue;
        strTag += " " + o.Key(i) + "=\"" + o(i) + "\"";
    }
    strTag += ">";
    WriteHTML(out,strTag);
}

function WriteTagEnd(o,out) {
    if (o("__container") == 1) {
        out.WriteText("</" + o("__class") + ">",0);
    }
}
function WriteTagEndVBS(o,out) {
    if (o("__container") == 1) {
        WriteHTML(out,"</" + o("__class") + ">");
    }
}
function CreateASPFileFromTree(tree,strFile,out) {
    compiler.ReportTrace(0,"CreateASPFileFromTree(" + strFile + ")");
    var o;
    
    // Pass 2: Place objects
    // Get new enum - the structure may be changed
    
    var e = new Enumerator(tree);
    
    // Pass 3: Place others
    e.moveFirst();
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        if (typeof(o) != "object") continue;
        compiler.ReportTrace(0,"Write element " + _writtenStmtTotal);
        _writtenStmtTotal ++;
        switch (o("Class")) {
            case "Text":
                out.WriteText(o("Content"),0);
            break;
            case "Script":
                out.WriteText("<% ",1);
                out.WriteText(o("Content"),1);
                out.WriteText(" %>",1);
            break;
            case "ASPDirective":
                out.WriteText("<% @Language=" + o("Language") + " %>",1);
            break;
            case "Assignment":
                out.WriteText("<%= " + o("Content") + " %>");
            break;
            case "Object":
                out.WriteText("<OBJECT RUNAT=\"SERVER\" PROGID=\"" + o("ProgID") + "\" + ID=\"" + o("ID") + "\"></OBJECT>",1);
            break;
            case "IncludeBegin":
                out.WriteText("<!-- Begin include file " + o("Value") + " -->",1);
            break;
            case "IncludeEnd":
                out.WriteText("<!-- End include file " + o("OuterFile") + " -->",1);
            break;
            case "Container":
                out.WriteText("<!-- Container Begin -->",1);
                    if (!CreateASPFileFromTree(o,strFile,out)) return false
                out.WriteText("<!-- Container End -->",1);
            break;
            case "Custom":
                // Skip
            break;
            default:
                // TAG
                WriteTagBegin(o,out);
                    if (!CreateASPFileFromTree(o,strFile,out)) return false
                WriteTagEnd(o,out);
        }
    }
    return true;
}
function CreateHTMLFileFromTree(tree,strFile,out) {
    compiler.ReportTrace(0,"CreateHTMLFileFromTree(" + strFile + ")");
    var o;
    
    var e = new Enumerator(tree);
    
    // Pass 3: Place others
    e.moveFirst();
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        if (typeof(o) != "object") continue;
        compiler.ReportTrace(0,"Write element " + _writtenStmtTotal);
        _writtenStmtTotal ++;
        switch (o("Class")) {
            case "Text":
                out.WriteText(o("Content"),0);
            break;
            case "IncludeBegin":
                out.WriteText("<!-- Begin include file " + o("Value") + "-->",1);
            break;
            case "IncludeEnd":
                out.WriteText("<!-- End include file " + o("OuterFile") + "-->",1);
            break;
            case "Container":
                out.WriteText("<!-- Container Begin -->",1);
                    if (!CreateHTMLFileFromTree(o,strFile,out)) return false
                out.WriteText("<!-- Container End -->",1);
            break;
            case "Custom":
            case "ASPDirective":
            case "Assignment":
            case "Object":
            case "Script":
                // Skip
            break;
            default:
                // TAG
                WriteTagBegin(o,out);
                    if (!CreateHTMLFileFromTree(o,strFile,out)) return false
                WriteTagEnd(o,out);
        }
    }
    return true;
}
function CreatePlainFileFromTree(tree,strFile,out) {
    compiler.ReportTrace(0,"CreatePlainFileFromTree(" + strFile + ")");
    var o;
    
    // Place objects
    // Get new enum - the structure may be changed
    
    var e = new Enumerator(tree);
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        if (typeof(o) != "object") continue;
        switch (o("Class")) {
            case "Object":
                out.WriteText("Public " + o("ID"),1);
                out.WriteText("Set " + o("ID") + " Server.CreateObject(\"" + o("ProgID") + "\")",1);
            break;
        }
    }
    
    // Pass 3: Place others
    e.moveFirst();
    for (; !e.atEnd(); e.moveNext()) {
        o = e.item();
        if (typeof(o) != "object") continue;
        compiler.ReportTrace(0,"Write element " + _writtenStmtTotal);
        _writtenStmtTotal ++;
        switch (o("Class")) {
            case "Text":
                WriteHTML(out,o("Content"));
            break;
            case "Script":
                out.WriteText(o("Content"),1);
            break;
            case "Assignment":
                out.WriteText(g_ResponseWrite + " " + o("Content"),1);
            break;
            case "IncludeBegin":
                out.WriteText("'#! file " + o("Value"),1);
            break;
            case "IncludeEnd":
                out.WriteText("'#! file " + o("OuterFile"),1);
            break;
            case "Comment":
                out.WriteText(g_ResponseWrite + " \" <!-- COMMENT DISCARDED --> \"",1);
            break;
            case "Container":
                out.WriteText("' Container Begin",1);                
                    if (!CreatePlainFileFromTree(o,strFile,out)) return false
                out.WriteText("' Container End",1);
            break;
            default:
                // TAG
                WriteTagBeginVBS(o,out);
                    if (!CreatePlainFileFromTree(o,strFile,out)) return false
                WriteTagEndVBS(o,out);
            
        }
    }
    return true;
}
function CreateASPPlainFileFromTree(tree,strFile,out) {
    out.WriteText("<%",1);
    var result = CreatePlainFileFromTree(tree,strFile,out);
    out.WriteText("%>",1);
    return result;
}






/////////////////////////////////////////// END OF ASP PARSING ///////////////////////////////////

function    DumpTree(r) {
    var i;
    var str = "";
    for (i = 1; i <= r.Count; i++) {
    	if ( typeof(r(i)) == "object") {
    	    switch (r(i)("__class")) {
	            case "HTMLEmbedSearcher.newObjects.VBStmtString":
	                str += r(i)("__text") + " ";
	            break;
	            case "HTMLEmbedSearcher.newObjects.VBStmtBrackets":
	                str += r(i)("Brackets").charAt(0) + " ";
	            break;
	            case "HTMLEmbedSearcher.newObjects.VBStmtCrack":
	                str += r(i)("__text") + " ";
	            break;
	        }
	        str += " " + DumpTree(r(i));
	        switch (r(i)("__class")) {
	            case "HTMLEmbedSearcher.newObjects.VBStmtBrackets":
	                str += r(i)("Brackets").charAt(1) + " ";
	            break;
	        }
	    } 
    }
    return str;
}

// VBS processing utility functions

/* Creates syntax tree for the passed statment
 *  stmt - string statement
 *  returns: syntax tree
 */
var _g_processedStatementsCount = 0;
function ParseVBStatement(stmt) {
    var str = new String(stmt);
    var s = STMTParser.ParseString(str);
    _g_processedStatementsCount ++;
    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount);
    return s;    
}



////////////////////////////////////////////////////////////////////////////



function AddSymbol(sym) {
    this.symbols[this.symbols.length] = sym;
}

function CollectSymbolName(el,index) {
    var i;
    var cur;
    var arr = new Array();
    var o = new Object;
    o.tokens = arr;
    o.type = 0; // 0 - not a symbol, 1 - symbol, 2 - error
    var expect = 1; // 0 - expect dot or end, 1 - expect token
    
    for (i = index; i <= el.Count; i++) {
        cur = el(i);
        if (typeof(cur) == "object") {
            if (expect == 0) {
                switch (cur("__content")) {
                    case "Operator":
                        if (cur("__text") == ".") {
                            // new indirection
                            expect = 1;
                            break;
                        }
                    default:
                        // anything else means end of the symbol
                        return o;
                }
            } else {
                switch (cur("__content")) {
                    case "Word":
                    case "NormalBrackets":
                        arr[arr.length] = el(i);
                        expect = 0;
                        o.type = 1;
                    break;
                    case "Keyword":
                        // Keyword can be a token if used after a dot
                        if (arr.length > 0) {
                            arr[arr.length] = el(i);
                            expect = 0;
                            o.type = 1;
                            break;
                        }
                    default:
                        // Not a symbol token
                        if (arr.length == 0) {
                            // Just not a symbol
                            return o; // Empty array means - not a symbol
                        } else {
                            // We have dot before
                            o.type = 2; // error
                            return o;
                        }
                }
            }
        }
    }
    // End of statement
    if (expect = 1) {
        // Error if not empty
        if (o.tokens.length > 0) o.type = 2;
    }
    return o;
}

// el - are the brackets containing the passed arguments
function ParseFunctionCallList(el) {
    if (el("__content") != "NormalBrackets") return null; // Not a function call
    var cur,i;
    var arr = new Array();    // Array with the arguments
    var arg = new ActiveXObject("newObjects.utilctls.VarDictionary");
    var s = "s";
    for (i = 1; i <= el.Count; i++) {
        cur = el(i);
        if (typeof(cur) == "object") {
            switch (cur("__content")) {
                case "Operator":
                    if (cur("__text") == ",") {
                        // argument separator
                        // Arguments should be checked outside
                        arr[arr.length] = arg;
                        arg = new ActiveXObject("newObjects.utilctls.VarDictionary");
                        break;
                    } 
                default:
                    arg.Add(s + i,cur);
            }
        }
    }
    arr[arr.length] = arg;
    return arr;
}

// el - are the brackets containing the passed arguments
// index - list start point
function ParseCommaList(el,index) {
    var cur,i;
    var arg = new ActiveXObject("newObjects.utilctls.VarDictionary");
    var arr = new Array();
    var s = "s";
    
    for (i = index; i <= el.Count; i++) {
        cur = el(i);
        if (cur("__content") == "Operator") {
            if (cur("__text") == ",") {
                arr[arr.length] = arg;
                arg = new ActiveXObject("newObjects.utilctls.VarDictionary");
                continue;
            }
        }
        // If not
        if (cur("__content") == "Keyword") {
            // Lists may end at these keywords
            switch (cur("__text")) {
                case "Else":
                case "Elseif":
                    arr[arr.length] = arg;
                    arg = new ActiveXObject("newObjects.utilctls.VarDictionary");
                    return arr;
                break;
            }
        }
        arg.Add(s + i,cur);    
    }
    arr[arr.length] = arg;
    return arr;
}

/* Creates array of arguments
 * every argument is represented as follows
 *          .name - arg name
 *          .type - 0 - variable, 1 - array
 *          .pass - 0 - byval, 1 - byref
 */
function ParseArgumentList(str,name) {
    var s = ParseVBStatement(str);
    var arr = new Array();
    var i,j;
    var cur;
    var last = null;
    
    if (s.Count > 0) {
        if (typeof(s(1)) != "object") {
            CR(0);
            compiler.ReportError(cERR_UnExpected,"In " + name + ": Unexpected structure of the argument list",0);
            compiler.ReportMessage(3,str);
        } else {
            if (s(1)("__content") != "NormalBrackets") {
                CR(0);
                compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": ( - expected in the argument list start.",0);
                compiler.ReportMessage(3,str);
            } else {
                // cycle through the arguments
                for (i = 1; i <= s(1).Count; i++) {
                    cur = s(1)(i);
                    if (typeof(cur) == "object") {
                        switch (cur("__content")) {
                            case "Keyword":
                                if (cur("__text") != "Byval" && cur("__text") != "Byref") {
                                    // Unexpected keyword
                                    CR(0);
                                    compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Unexpected keyword (" + cur("__text") + ") in the argument list",0);
                                    compiler.ReportMessage(3,str);
                                } else {
                                    if (last != null) {
                                        // It is an error
                                        CR(0);
                                        compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error - " + cur("__text") + " after the argument name",0);
                                        compiler.ReportMessage(3,str);
                                    } else {
                                        last = new Object;
                                        arr[arr.length] = last;
                                        if (cur("__text") != "Byref") {
                                            last.pass = 0;
                                        } else {
                                            last.pass = 1;
                                        }
                                        last.type = 0;
                                    }
                                }
                            break;
                            case "Word":
                                // A word 
                                if (last == null) {
                                    // first argument
                                    last = new Object;
                                    arr[arr.length] = last;
                                    last.name = cur("__text");
                                    last.type = 0;
                                    last.pass = 1;
                                } else {
                                    // If the name is already defined - this is an error
                                    if (typeof(last.name) != "undefined") {
                                        CR(0);
                                        compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error in the argument list",0);
                                        compiler.ReportMessage(3,str);
                                    } else {
                                        last.name = cur("__text");
                                        last.type = 0;
                                    }
                                }
                            break;
                            case "NormalBrackets":
                                if (last == null) {
                                    CR(0);
                                    compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error in the argument list",0);
                                    compiler.ReportMessage(3,str);
                                } else {
                                    if (typeof(last.name) == "undefined") {
                                        CR(0);
                                        compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error in the argument list  - subscript without argument name",0);
                                        compiler.ReportMessage(3,str);
                                    } else {
                                        // check to see if the list is empty
                                        last.type = 1;
                                        for (j = 1; j <= cur.Count; j++) {
                                            if (typeof(cur(j)) == "object") {
                                                CR(0);
                                                compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error in the argument list - argument subscript not empty",0);
                                                compiler.ReportMessage(3,str);
                                            }
                                        }
                                    }
                                }
                            break;
                            case "Operator":
                                if (cur("__text") == ",") {
                                    // End of the element
                                    // Check to see if it is correct
                                    if (typeof(last.name) == "undefined") {
                                        CR(0);
                                        compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Syntax error in the argument list",0);
                                        compiler.ReportMessage(3,str);
                                    }
                                    last = null;
                                } else {
                                    // No other operators are allowed
                                    compiler.ReportError(cERR_ArgListSyntax,"In " + name + ": Unexpected operator in the argument list",0);
                                    compiler.ReportMessage(3,str);
                                    return false;
                                }
                            break;
                        }
                    }
                }
            }
        }
    }
    return arr;
}


// Universal function for dim statments
/* Creates array of the declared variables
 *  s   - syntax tree
 *  c   - constants pool ( *** not used currently)
 *  returns: object
 *                  .name - var name
 *                  .type - variable, array
 *                  .subscripts - if any - array of numbers
 *                  .access - Public, Private
 */
function TraceDimStatement(node,capt,bDiscardDuplicates) {
    var s = ParseVBStatement(node("Dim") + " " + node("__text"));
    var i;
    var cur,n = -1; // general vars
    var j,last,subs; // temporary vars
    var arr = this.symbols;
    var access = "Public";
    
    if (node("Dim") == "Private") {
        access = "Private";
    }
    for (i = 2; i <= s.Count; i++) {
        cur = s(i);
        switch (cur("__content")) {
            case "NumericLiteral":
                CR(0);
                compiler.ReportError(cERR_DIMSyntax,"In Class " + capt + ": Numeric literals are not allowed in variable declaration statments",0);
                compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                return false;
            break;
            case "Keyword":
                CR(0);
                compiler.ReportError(cERR_DIMSyntax,"In Class " + capt + ": Keyword in a variable declaration statment",0);
                compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                return false;
            break;
            case "Operator":
                if (cur("__text") == ",") {
                    n = -1; // End of the element
                } else {
                    // No other operators are allowed
                    CR(0);
                    compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Unexpected operator in the variable declaration list",0);
                    compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                    return false;
                }
            break;
            case "Word":
                // A word 
                if (n >= 0) {
                    // comma missing
                    CR(0);
                    compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Comma expected in the variable declaration statment",0);
                    compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                    return false;
                } else {
                    if (bDiscardDuplicates) {
                        if (this.FindSymbol(cur("__text")) != null) {
                            compiler.ReportWarning(cERR_DuplicateNames,"Variable (" + + ") matches existing symbol in " + capt + ". Discarding the new declaration.",0);
                            CR(-1);
                        }
                    } else {
                        n = arr.length; // The last element in the comma sep. list
                        arr[n] = new VBSymbol(this);
                        arr[n].name = cur("__text");
                        arr[n].type = "Variable";
                        arr[n].access = access;
                    }
                }
            break;
            case "TrailingComment":
                i = s.Count; // Exit for
            break;
            case "NormalBrackets":
                if (n < 0) {
                    // Expected identifier
                    CR(0);
                    compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Identifier expected in the variable declaration statment",0);
                    compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                    return false;
                } else {
                    // Extract the subscripts without checking
                    last = -1; // -1 nothing, 0  number, 1 comma
                    arr[n].subscripts = new Array;
                    arr[n].type = "Array";
                    subs = arr[n].subscripts;
                    for (j = 1; j <= cur.Count; j++) {
                        if (typeof(cur(j)) == "object") {
                            switch (cur(j)("__content")) {
                                case "NumericLiteral":
                                    if (last != 0) {
                                        last = 0;
                                        subs[subs.length] = cur(j)("__text");
                                    } else {
                                        CR(0);
                                        compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Syntax error in the array subscripts",0);
                                        compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                                        return false;
                                    }
                                break;
                                case "Operator":
                                    if (cur(j)("__text") != ",") {
                                        CR(0);
                                        compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Orperator [" + cur(j)("__text") + "] not allowed in the array subscript",0);
                                        compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                                        return false;
                                    } else {
                                        if (last != 0) {
                                            CR(0);
                                            compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Syntax error in the array subscripts - comma in wrong place",0);
                                            compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                                            return false;
                                        } else {
                                            last = 1;
                                        }
                                    }
                                break;
                                default:
                                    CR(0);
                                    compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Syntax error in the array subscripts - unknown entry",0);
                                    compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                                    return false;
                            }
                            
                        }
                    }
                }
            break;
            case "SquareBrackets":
                CR(0);
                compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Square brackets [] not allowed",0);
                compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                return false;
            break;
            case "StringLiteral":
                CR(0);
                compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": String literals are not allowed in variable declaration statments",0);
                compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                return false;
            break;
            default:
                // Unknown entry
                CR(0);
                compiler.ReportError(cERR_DIMSyntax,"In " + capt + ": Syntax error in the variable declaration statment",0);
                compiler.ReportMessage(3,node("Dim") + " " + node("__text"));
                return false;
        }
    }
    if (arr.length > 0) {
        return true;
    } else {
        return false;
    }
}

function BuildMethodSymbolTable(node,capt) {
    var el;
    var i;
    var lastDirective = null;
    for (i = 1; i <= node.Count; i++) {
        el = node(i);
        if (typeof(el) == "object") {
            switch (el("__content")) {
                case "Dim":
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "DbgTrace") {
                            lastDirective = null;
                            break;
                        }
                    }
                    if (!this.TraceDimStatement(el,capt)) return false;
                    lastDirective = null;
                break;
                case "Statement":
                    // nothing
                break;
                case "Comment":
                    // nothing
                break;
                case "Directive":
                    lastDirective = el;
                break;
                case "Const": 
                    this.AddConstants(el,capt);
                break;
                case "With":
                    // recurse
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "DbgTrace") {
                            lastDirective = null;
                            break;
                        }
                    }
                    if (!this.BuildMethodSymbolTable(el,capt)) return false;
                    lastDirective = null;
                break;
                default:
                    CR(0);
                    Compiler.ReportError(cERR_cErr_UnexpectedStatement,"In " + capt + ": Unexpected statement in function/sub/property body.",0);
                    compiler.ReportMessage(3,"Statement type: " + node("__content"));
                    return false;
            }
        }
    }
    return true;
}

function AddConstants(el,capt) {
    var tree = ParseVBStatement(el("__text"));
    var i = 1;
    var sym;
    
    var name,value,expect = 1;
    
    do {
        if (i > tree.Count) {
            if (expect == 0) return true;
            CR(0);
            compiler.ReportError(cErr_SyntaxError,"Syntax error in constant statement in the declaration of " + capt,0);
            compiler.ReportMessage(3,el("__text"));
            return false;
        }
        switch(expect) {
            case 1: // name
                if (tree(i)("__content") == "Word") {
                    name = tree(i)("__text");
                    expect = 2;
                } else {
                    CR(0);
                    compiler.ReportError(cErr_SyntaxError,"Syntax error in constant statement in the declaration of " + capt,0);
                    compiler.ReportMessage(3,el("__text"));
                    return false;
                }
            break;
            case 2: // assignment
                if (tree(i)("__content") == "Operator" && tree(i)("__text") == "=") {
                    expect = 3;
                } else {
                    CR(0);
                    compiler.ReportError(cErr_SyntaxError,"Unexpected operator in constant statement in the declaration of " + capt,0);
                    compiler.ReportMessage(3,el("__text"));
                    return false;
                }
            break;
            case 3: // value
                if ( (tree(i)("__content") == "NumericLiteral") || (tree(i)("__content") == "StringLiteral")) {
                    value = tree(i)("__text");
                    // Check if it exists
                    if (this.FindSymbol(name) != null) {
                        compiler.ReportWarning(cERR_DuplicateNames,"Constant (" + name + ") matches existing symbol in " + capt + ". Discarding the new declaration.",0);
                        CR(-1);
                        expect = 0;
                        break;
                    }
                    // Create and add a symbol
                    sym = new VBSymbol(this);
                    sym.type = "Const";
                    sym.name = name;
                    if (el("Access") == "Private") {
                        sym.access = "Private";
                    } else {
                        sym.access = "Public";
                    }
                    sym.value = value;
                    this.symbols[this.symbols.length] = sym;
                    expect = 0;
                } else {
                    CR(0);
                    compiler.ReportError(cErr_SyntaxError,"Syntax error in constant statement in the declaration of " + capt,0);
                    compiler.ReportMessage(3,el("__text"));
                    compiler.ReportMessage(3,"Assigning: " + tree(i)("__content") + " " + tree(i)("__text"));
                    return false;
                }
            break;
            default: // nothing special
                // Can be only comma
                if ( (tree(i)("__content") == "Operator") && (tree(i)("__text") == ",")) {
                    expect = 1;
                } else {
                    CR(0);
                    compiler.ReportError(cErr_SyntaxError,"Syntax error in constant statement in the declaration of " + capt,0);
                    compiler.ReportMessage(3,el("__text"));
                    return false;
                }
        }
         
        i++;        
    } while(true);
    return true;
}

/* Creates an entry for the method in the class
 * Entry definition:
 *  VBSymbol = {
 *      name - member name
 *      type - Function, Sub, Set (property set), Get (property Get), Let (property Let)
 *      access - Public, Private
 *      arguments - array of vardecls
 *      body - syntax tree of the body
 *  };
 */
function AddMethod(el,bDiscardDuplicates) {
    var o = new VBSymbol(this);
    if (el("__class") == "HTMLEmbedSearcher.newObjects.VBStmtSrch") {
        switch (el("__content")) {
            case "Function":
                
                
                if (el("Access") != "Private") {
                    o.access = "Public";    
                } else {
                    o.access = el("Access");
                }
                o.type = "Function";    
                o.name = new String(el("FunctionName"));
                o.body = el;
                o.arguments = ParseArgumentList(el("__text"),this.name + "." + o.name);
                o.BuildMethodSymbolTable = BuildMethodSymbolTable;
                o.BuildMethodSymbolTable(el,this.name + "." + o.name);
            break;
            case "Sub":
            
            
                if (el("Access") != "Private") {
                    o.access = "Public";    
                } else {
                    o.access = el("Access");
                }
                o.type = "Sub";    
                o.name = new String(el("FunctionName"));
                o.body = el;
                o.arguments = ParseArgumentList(el("__text"),this.name + "." + o.name);
                
                if (o.name.toUpperCase() == "CLASS_INITIALIZE") {
                    // constructor
                    
                    // check the count of arguments
                    if (o.arguments.length > 0) {
                        CR(0);
                        compiler.ReportError(cERR_ConstructorArgs,"In class " + this.name + "." + o.name + ": Constructor cannot have arguments",0);
                    }
                    if (o.access != "Private") {
                        CR(0);
                        compiler.ReportError(cERR_ConstructorAccess,"In class " + this.name + "." + o.name + ": Constructors must be declared Private",0);
                    }
                    
                    // do not change the name
                    // o.name = this.name + "_" + "Initialize";
                }
                if (o.name.toUpperCase() == "CLASS_TERMINATE") {
                    // constructor
                    
                    // check the count of arguments
                    if (o.arguments.length > 0) {
                        CR(0);
                        compiler.ReportError(cERR_DestructorArgs,"In class " + this.name + "." + o.name + ": Destructor cannot have arguments",0);
                    }
                    if (o.access != "Private") {
                        CR(0);
                        compiler.ReportError(cERR_DestructorAccess,"In class " + this.name + "." + o.name + ": Destructor must be declared Private",0);
                    }
                    
                    // do not change the name
                    // o.name = this.name + "_" + "Terminate";
                }
                o.BuildMethodSymbolTable = BuildMethodSymbolTable;
                o.BuildMethodSymbolTable(el,this.name + "." + o.name);
            break;
            case "Property":
            
            
                if (el("Access") != "Private") {
                    o.access = "Public";    
                } else {
                    o.access = el("Access");
                }
                o.type = el("PropType");    
                o.name = new String(el("FunctionName"));
                o.body = el;
                o.isProperty = true;
                o.arguments = ParseArgumentList(el("__text"),this.name + "." + o.name);
                // Check for common errors
                if (o.arguments.length == 0) {
                    if (o.type == "Let" || o.type == "Set") {
                        CR(0);
                        compiler.ReportError(cERR_SetLetRwuiredArgs,"In " + this.name + "." + o.name + ": Let and Set properties should have at least one argument",0);
                    }
                }
                if (o.type != "Let" && o.type != "Get" && o.type != "Set") {
                    CR(0);
                    compiler.ReportError(cERR_PropertySyntax,"In " + this.name + "." + o.name + ": Unknown keyword in the property definition",0);
                }
                o.BuildMethodSymbolTable = BuildMethodSymbolTable;
                o.BuildMethodSymbolTable(el,this.name + "." + o.name);
            break;
        }
    }
    if (typeof(o.name) != "undefined") {
        // Add it
        o.CheckMembers(this.name + "." + o.name);
        if (bDiscardDuplicates) {
            if (this.FindSymbol(o.name,o.type) != null) {
                compiler.ReportWarning(cERR_DuplicateNames,o.type + " " + o.name + " duplicated in " + this.name + ". Discarding the new definition.",0);
            } else {
                this.symbols[this.symbols.length] = o;
            }
        } else {
            this.symbols[this.symbols.length] = o;
        }
    }
}

/* Searches for symbol by name
 * returns: the object representing the symbol or null if not found
 *      
 */
function FindSymbol(nameIN,Verb) {
    var i;
    var name = new String(nameIN);
    name = name.toUpperCase();
    var sym;
    for (i = 0; i < this.symbols.length; i++) {
        if (this.symbols[i].name.toUpperCase() == name) {
            sym = this.symbols[i];
            if (sym.isProperty) {
                if (typeof(Verb) == "string") {
                    if (sym.type == Verb) return sym;
                    continue;
                } 
            }
            return sym;
        } 
    }
    return null;
}

function CheckMembers(loc) {
    // Check for duplicate symbols and reserved names
    var i,j;
    var name;
    for (i = 0; i < this.symbols.length; i++) {
        name = this.symbols[i].name;
        if (name.toUpperCase() == project.contextName.toUpperCase()) {
            if (this.type == "Class" && this.type == "Project") {
                CR(0);
                compiler.ReportError(cERR_ReservedName,"In " + loc + ": " + project.contextName + " name is used but it is reserved.",0);
            } else {
                CR(-1);
                compiler.ReportError(cERR_ReservedName,"In " + loc + ": " + project.contextName + " name is used but it is reserved.",0);
            }
        }
        if (!this.isRoot) project.CheckReserved(name,loc);
        for (j = i + 1; j < this.symbols.length; j++) {
            if (name.toUpperCase() == this.symbols[j].name.toUpperCase()) {
                if (this.symbols[i].isProperty) {
                    if (!this.symbols[j].isProperty) {
                        CR(0);
                        compiler.ReportError(cERR_DuplicateNames,"In " + loc + ": " + name + " is duplicated.",0);
                    } else {
                        if (this.symbols[j].type == this.symbols[i].type) {
                            CR(0);
                            compiler.ReportError(cERR_DuplicateNames,"In " + loc + ": property " + this.symbols[j].type + " " + name + " is duplicated.",0);    
                        }
                    }
                } else {
                    CR(0);
                    compiler.ReportError(cERR_DuplicateNames,"In " + loc + ": " + name + " is duplicated.",0);
                }
            }
        }
        for (j = 0; j < this.arguments.length; j++) {
            if (name.toUpperCase() == this.arguments[j].name.toUpperCase()) {
                CR(0);
                compiler.ReportError(cERR_DuplicateNames,"In " + loc + ": local variable " + name + " is duplicated in the argument list.",0);
            }
        }
    }
    for (i = 0; i < this.arguments.length; i++) {
        name = this.arguments[i].name;
        project.CheckReserved(name,loc);
        for (j = i + 1; j < this.arguments.length; j++) {
            if (name.toUpperCase() == this.arguments[j].name.toUpperCase()) {
                CR(0);
                compiler.ReportError(cERR_DuplicateNames,"In " + loc + ": argument " + name + " is duplicated.",0);
            }
        }
    }
}


// Symbol table object
/* 
    .name           -   (string) name of the symbol
    .type           -   (string) type of the symbol
                            Values: Variable, Array, Function, Sub, Let, Get, Set
    .access         -   (string) access type of the symbol
                            Values: Public, Private
    .arguments      -   (array)  arguments list
    .body           -   (object) body syntax tree
    .subscripts     -   (array)  array of numbers - subscripts
    .symbols        -   (array)  array of local symbols
    .isProperty     -   (boolean) id it property definition
 */

////////////////////////////////////////// VBSClass ////////////////////////////////////////////////////

// Class representing the VBS class object
// =======================================
function VBSClass(project,name,dstFile,srcFile) {
    this.name           = name;
    this.symbols        = new Array();
    this.autosymbols    = new Array();
    this.type           = "Class";
    this.access         = "Public";
    this.arguments      = new Array();
    this.body           = null;
    this.subscripts     = null;
    this.isProperty     = false;
    this.parent         = project;
    this.isRoot         = false;
    this.PublicType     = 0;
    this.autovar        = false;
    
    
    this.project = project; // reference to the project
    this.dstFile = dstFile;
    this.srcFile = srcFile;    
}

VBSClass.prototype.AddSymbol = AddSymbol;
VBSClass.prototype.AddConstants = AddConstants;
VBSClass.prototype.TraceDimStatement = TraceDimStatement;
VBSClass.prototype.FindSymbol = FindSymbol;
VBSClass.prototype.AddMethod = AddMethod;
VBSClass.prototype.CheckMembers = CheckMembers;
VBSClass.prototype.AddAutoSymbol = function(s) {
    var name = new String(s);
    name = name.toUpperCase();
    for (i = 0; i < this.autosymbols.length; i++) {
        if (this.autosymbols[i].name.toUpperCase() == name) {
            return;
        } 
    }
    o = new VBSymbol(this);
    o.name = s;
    o.type = "Variable";
    o.access = "Public";
    this.autosymbols[this.autosymbols.length] = o;
    CR(-1);
    compiler.ReportWarning(cERR_UndeclaredSymbol,"Declaring automatically (" + s + ") in the class " + this.name,0);
    //compiler.ReportMessage(3,"Declaring automatically (" + s + ") in the class " + this.name);
}


VBSClass.prototype.LinkSymbol = function(s,name) {
    var sym = this.FindSymbol(s);
    var o;
    if (sym == null) {
        // Search root class
        if (!this.isRoot) {
            sym = this.project.Root.FindSymbol(s);
            if (sym == null) {
                // Not found there - search globals
                if (this.autovar) {
                    if (this.project.IsLinkable(s)) {
                        return this.project.LinkSymbol(s,name);
                    } else {
                        this.AddAutoSymbol(s);
                        return s;
                    }
                } else {
                    return this.project.LinkSymbol(s,name);
                }
            } else {
                // Symbol is in the Root class - redirect to it
                if (sym.access != "Public") {
                    CR(0);
                    compiler.ReportError(cERR_Unaccessible,"In " + name + " cannot access non-public member (" + s + ") of the root class",0);
                } 
                return this.project.contextName + "." + s;
            }
        } else {
            if (this.autovar) {
                if (this.project.IsLinkable(s)) {
                    return this.project.LinkSymbol(s,name);
                } else {
                    this.AddAutoSymbol(s);
                    return s;
                }
            } else {
                return this.project.LinkSymbol(s,name);
            }
        }
    } else {
        // Found
        return s;
    }
}

VBSClass.prototype.Analyse = function(node) {
    var e;
    var i;
    var o; // temp var
    
    this.body = node;
    this.name = node("ClassName");
    this.dstFile = CombinePaths(vbOutputPath,this.name + ".cls");
                

    // Analyze the statements
    e = new Enumerator(node);
    for (;!e.atEnd();e.moveNext()) {
        //i = node(e.item());
        i = e.item();
        if (typeof(i) == "object") {
            switch (i("__content")) {
                case "Dim":
                    this.TraceDimStatement(i,this.name);
                break;
                case "Function":
                case "Sub":
                case "Property":
                    this.AddMethod(i);
                break;
                case "Comment":
                    continue; // Comments are discarded
                break;
                default:
                    compiler.ReportError(cErr_UnexpectedStatement,"Ilegal statement in the class definition (" + this.name + ")",0);
                    CR(0);
                    compiler.ReportMessage(3,"Illegal statement type: " + i("__content"));
                    if (i("__content") == "Statement") compiler.ReportMessage(3,"Statement: " + i("__text"));
            }
        }
    }
    
    // Check the members
    this.CheckMembers("class " + this.name);
    
    // Create context symbol
    e = new VBSymbol(this);
    e.type = "Variable";
    e.access = "Public";
    e.name = this.project.contextName;
    this.symbols[this.symbols.length] = e;
    
    return true;
}
    
// Does output code generation
// Supposes no errors found
VBSClass.prototype.Generate = function() {
    var cls; // Output file
    cls = fso.CreateFile(this.dstFile,0x1012);
    
    if (this.isRoot) {
        cls.WriteText("VERSION 1.0 CLASS",1);
        cls.WriteText("BEGIN",1);
        cls.WriteText("  MultiUse = -1  'True",1);
        cls.WriteText("  Persistable = 0  'NotPersistable",1);
        cls.WriteText("  DataBindingBehavior = 0  'vbNone",1);
        cls.WriteText("  DataSourceBehavior  = 0  'vbNone",1);
        cls.WriteText("  MTSTransactionMode  = 0  'NotAnMTSObject",1);
        cls.WriteText("END",1);
        cls.WriteText("Attribute VB_Name = \"" + this.name + "\"",1);
        cls.WriteText("Attribute VB_GlobalNameSpace = False",1);
        cls.WriteText("Attribute VB_Creatable = True",1);
        cls.WriteText("Attribute VB_PredeclaredId = False",1);
        cls.WriteText("Attribute VB_Exposed = True",1);
        cls.WriteText("",1);
    } else {
        // Common attributes
        cls.WriteText("VERSION 1.0 CLASS",1);
        cls.WriteText("BEGIN",1);
        cls.WriteText("  MultiUse = -1  'True",1);
        cls.WriteText("  Persistable = 0  'NotPersistable",1);
        cls.WriteText("  DataBindingBehavior = 0  'vbNone",1);
        cls.WriteText("  DataSourceBehavior  = 0  'vbNone",1);
        cls.WriteText("  MTSTransactionMode  = 0  'NotAnMTSObject",1);
        cls.WriteText("END",1);
        cls.WriteText("Attribute VB_Name = \"" + this.name + "\"",1);
        cls.WriteText("Attribute VB_GlobalNameSpace = False",1);
        if (this.PublicType == 2) {
            cls.WriteText("Attribute VB_Creatable = True",1);
        } else {
            cls.WriteText("Attribute VB_Creatable = False",1);
        }
        cls.WriteText("Attribute VB_PredeclaredId = False",1);
        if (this.PublicType >= 1) {
            cls.WriteText("Attribute VB_Exposed = True",1);
        } else {
            cls.WriteText("Attribute VB_Exposed = False",1);
        }
        cls.WriteText("",1);    
    }
    
    
    // Reports
    compiler.ReportFile(cFileVB,this.dstFile,"VB class: " + this.name);
    compiler.ReportFile(cFileHidden,this.srcFile,"Class declaration file");    
    
    var i,j;    // Misc counters
    var el;     // Element temp variable
    var arg;    // Argument temp variable
    var str;    // Statements buffer
    // Begin generation
    // Add variables
    cls.WriteText("' Class variables",1);
    for (i = 0; i < this.symbols.length; i++) {
        el = this.symbols[i];
        switch (el.type) {
            case "Variable":
            case "Array":
            case "Const":
                if (el.type == "Array") {
                    if (el.access != "Private") {
                        el.access = "Private";
                        CR(-1);
                        compiler.ReportWarning(cERR_Unaccessible,"In class " + this.name + " array (" + el.name + ") is declared public. Changing to Private, but this may cause your application to run incorrectly.",0);
                    }
                }
                el.Generate(cls);
            break;
        }
    }
    
    // Add members
    cls.WriteText("' Class members",1);
    for (i = 0; i < this.symbols.length; i++) {
        el = this.symbols[i];
        switch (el.type) {
            case "Function":
            case "Sub":
            case "Let":
            case "Get":
            case "Set":
                el.Generate(cls);
            break;
        }
    }
    
    // Automaticlly created variables if any
    if (this.autosymbols.length > 0) {
        cls.WriteText("' Automatically created class variables",1);
        for (i = 0; i < this.autosymbols.length; i++) {
            el = this.autosymbols[i];
            switch (el.type) {
                case "Variable":
                case "Array":
                case "Const":
                    if (el.type == "Array") {
                        if (el.access != "Private") {
                            el.access = "Private";
                            CR(-1);
                            compiler.ReportWarning(cERR_Unaccessible,"In class " + this.name + " array (" + el.name + ") is declared public. Changing to Private, but this may cause your application to run incorrectly.",0);
                        }
                    }
                    el.Generate(cls);
                break;
            }
        }
    }
    
    return true;
}
    



//////////////////////////////////////// VBSymbol /////////////////////////////////////////////

function VBSymbol(parent) {
    this.parent     = parent;
    this.isProperty = false;
    this.type       = null;
    this.access     = "Public";
    this.arguments  = new Array();
    this.body       = null;
    this.subscripts = new Array();
    this.symbols    = new Array();
}
VBSymbol.prototype.TraceDimStatement   = TraceDimStatement;
VBSymbol.prototype.AddSymbol = AddSymbol;
VBSymbol.prototype.AddConstants = AddConstants;
VBSymbol.prototype.CheckMembers = CheckMembers;
VBSymbol.prototype.FindSymbol = FindSymbol;
VBSymbol.prototype.GenerateDeclaration = function(file) {
    return true;
}
VBSymbol.prototype.Generate = function(file) {
    // Does something for the symbels having body
    var     i,j;
    var     el;
    var     str,s;
    var     arg;
    
    switch (this.type) {
        case "Sub":
        case "Function":
        case "Set":
        case "Let":
        case "Get":
            // Add header
            str = this.access + " ";
            if (this.isProperty) {
                str += "Property ";
            }
            str += this.type + " " + this.name + "(";
            if (this.arguments.length > 0) {
                // Process the argument list
                for (j = 0; j < this.arguments.length; j++) {
                    arg = this.arguments[j];
                    if (arg.pass == 1) {
                        str += "ByRef ";
                    } else {
                        str += "ByVal ";
                    }
                    str += arg.name;
                    if (arg.type == 1) {
                        str += "()";
                    }
                    if (j < (this.arguments.length - 1)) {
                        str += ",";
                    }
                }
            } 
            str += ")";
            file.WriteText(str,1);
                
            // Add body
            //////////////////////
                
            if (this.symbols != null) {
                // Add local variables
                if (this.symbols.length > 0) file.WriteText("    ' Local variables (" + this.symbols.length + ")",1);
                for (i = 0; i < this.symbols.length; i++) {
                    el = this.symbols[i];
                    if (!el.Generate(file)) return false;
                }
            }
            
            // Generate statements
            file.WriteText("    ' Code",1);
            this.GenerateCode(this.body,this.parent.name + "." + this.name,null,file);

            // Add footer
            ///////////////////////////
            str = "End ";
            if (this.isProperty) {
                str += "Property";
            } else {
                switch (this.type) {
                    case "Sub": 
                        str += "Sub";
                    break;
                    case "Function":
                        str += "Function";
                    break;
                }
            }
            file.WriteText(str,1);
        break;
        case "Variable":
        case "Array":
            // Select declaration to use depending on the parent type
            switch (this.parent.type) {
                case "Class":
                    str = this.access + " " + this.name;
                break;
                case "Sub":
                case "Get":
                case "Function":
                case "Set":
                case "Let":
                    str = "    Dim " + this.name;
                break;
                case "Project":
                    str = "Public " + this.name;
                break;
            }
            if (this.type == "Array") {
                str += "(";
                    for (j = 0; j < this.subscripts.length; j++) {
                        str += this.subscripts[j];
                        if (j < (this.subscripts.length - 1)) {
                            str += ",";
                        }
                    }
                str += ")";
            } 
            file.WriteText(str,1);
        break;
        case "Const":
            switch (this.parent.type) {
                case "Class":
                    file.WriteText("Const " + this.name + " = " + this.value,1);
                break;
                case "Sub":
                case "Get":
                case "Function":
                case "Set":
                case "Let":
                    file.WriteText("    Const " + this.name + " = " + this.value,1);
                break;
                case "Project":
                    file.WriteText("Public Const " + this.name + " = " + this.value,1);
                break;
            }
        break;
        case "Class":
            // Add header
            //////////////////////
            file.WriteText("Class " + this.name,1);
                
            // Add body
            //////////////////////
            if (this.symbols != null) {
                // Add local variables 
                file.WriteText("    ' Local variables",1);
                for (i = 0; i < this.symbols.length; i++) {
                    el = this.symbols[i];
                    if (el.type == "Variable" || el.type == "Array") {
                        if (!el.Generate(file)) return false;
                    }
                }
                // Generate routines
                file.WriteText("    ' Code",1);
                for (i = 0; i < this.symbols.length; i++) {
                    el = this.symbols[i];
                    if (el.type != "Variable" && el.type != "Array") {
                        if (!el.Generate(file)) return false;
                    }
                }
            }
            // Add footer
            ///////////////////////////
            file.WriteText("End Class",1);
        break;
    }
    return true;
}
// Cycles through the statements of the procedure
VBSymbol.prototype.GenerateCode = function(t,name,wBlock,file,numStatement) {
    var i;
    var s;
    var el;
    var lastDirective = null;
    var nStatement = (numStatement == null)?1:numStatement;
    
    for (i = 1; i <= t.Count; i++) {
        if (typeof(t(i)) == "object") {
            
            if (t(i)("__content") == "With") {
                // Recurse
                nStatement++;
                if (lastDirective != null) {
                    if (lastDirective("Type") == "DbgTrace") {
                        lastDirective = null;
                        break;
                    }
                }
                // Header                
                el = ParseVBStatement(t(i)("__text"));
                s = this.GenerateStatement(el,name + ":" + nStatement,wBlock,0);
                file.WriteText("    With " + s,1);
                
                nStatement = this.GenerateCode(t(i),name,Trim(t(i)("__text")),file,nStatement);
                
                // Footer
                nStatement++;
                file.WriteText("    End With",1);
                lastDirective = null;
            } else {
                // Generate a statement
                switch (t(i)("__content")) {
                    case "Dim":
                        // already processed
                        nStatement++;
                    break;
                    case "Statement":
                        // Generate it
                        if (lastDirective != null) {
                            if (lastDirective("Type") == "DbgTrace") {
                                nStatement++;
                                lastDirective = null;
                                break;
                            }
                        }
                        el = ParseVBStatement(t(i)("__text"));
                        s = this.GenerateStatement(el,name + ":" + nStatement,wBlock,0);
                        if (s == null) {
                            file.WriteText("' ERROR found on this line",1);
                            return false;
                        }
                        file.WriteText(s,1);
                        nStatement++;
                        lastDirective = null;
                    break;
                    case "Comment":
                        // Discard it
                        nStatement++;
                    break;
                    case "Directive":
                        lastDirective = t(i);                        
                        nStatement++;
                    break;
                    case "Const":
                        // Already processed
                        nStatement++;
                    break;
                    default:
                        // Not allowed
                        CR(0);
                        compiler.ReportError(cErr_UnexpectedStatement,"In " + name + ": Unexpected statement type (" + t(i)("__content") + ")",0);
                        return false;
                }
                
            }
        }
    }
    return true;
} 
VBSymbol.prototype.FindArg = function(s) {
    var i;
    for (i = 0; i < this.arguments.length; i++) {
        if (this.arguments[i].name.toUpperCase() == s.toUpperCase()) return this.arguments[i];
    }
    return null;
}
// This member is intended to replace the symbol with the appropriate 
// reference 
VBSymbol.prototype.LinkSymbol = function(s,name) {
    var sym = this.FindSymbol(s);
    if (sym == null) {
        // Look in the argument list
        sym = this.FindArg(s);
        if (sym == null) {
            // Look in the parent
            return this.parent.LinkSymbol(s,name);
        } else {
            return s;
        }
    } else {
        return s;
    }
}
// Generates text of the statement
// Reads symbols and asks for replacements
/*  t       - syntax tree
 *                  stmt    - up to this point
 *  name    - namespace for errors
 *  wBlock  - With block (null if none)
 *  nesting - brackets if > 0
 */

// prev - values
// null - nothing
// 1    - word
// 2    - dotmark
// 3    - brackets
// 4    - other operator
// 5    - keyword
// 6    - literal

VBSymbol.prototype.GenerateStatement = function(t,name,wBlock,nesting) {
    // t is the syntax tree
    var stmt = (nesting == 0)?"    ":"";
    var i;
    var cur = null;
    var prev = null; // previous item
    var strTemp = "";
    var lastNew = -1;
    var strClassInit = "";
    var boolSetStatement = false;
    var boolSymbolIndexing = false;
    
    // prev = 0; // 0 - nothing, 1 - operator
    var boolPossileSubCall = (nesting == 0)?true:false;
    
    for (i = 1; i <= t.count; i++) {
        compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i);
        if (typeof(t(i)) == "object") {
            cur = t(i);
            switch (cur("__content")) {
                case "StringLiteral":
                case "NumericLiteral":
                compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Literal");
                    boolSymbolIndexing = false;
                    if (prev != 4 && prev != 5 && prev != null && !boolPossileSubCall) {
                        // It is an error
                        if (cur("__content") == "NumericLiteral") {
                            // check for signs
                            strTemp = new String(cur("__text"));
                            if (strTemp.charAt(0) == "+" || strTemp.charAt(0) == "-") {
                                // It is still possible that it is ok
                                if (prev == 1 || prev == 3 || prev == 6) {
                                    stmt += " " + strTemp.charAt(0);
                                    stmt += " " + strTemp.slice(1);
                                    prev = 6;
                                    break;
                                }
                            }
                        }
                        CR(0);
                        compiler.ReportError(cErr_SyntaxError,".In " + name + ": Syntax error - string or numeric literal is unexpected.",0);
                        compiler.ReportMessage(3,DumpTree(t));
                        return null;
                    } else {
                        if (prev == null) {
                            boolPossileSubCall = false;
                            if (nesting > 0) {
                                // allowed
                                stmt += " " + cur("__text");
                            } else {
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": Syntax error - string or numeric literal is unexpected.",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            }
                        } else {
                            boolPossileSubCall = false;
                            stmt += " " + cur("__text");
                        }
                    }
                    prev = 6; // literal
                break;
                case "NormalBrackets":
                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Brackets");
                    // recurse
                    // Any potential symbol ends here
                    switch (prev) {
                        case 1: // word ok
                            // Put it without checking
                            strTemp =  this.GenerateStatement(cur,name,wBlock,nesting + 1);
                            if (strTemp == null) return null;
                            stmt += "(" + strTemp + ")";
                            boolSymbolIndexing = true;
                        break;
                        case 2: // dotmark - error
                            CR(0);
                            compiler.ReportError(cErr_SyntaxError,"In " + name + ": . unexpected",0);
                            compiler.ReportMessage(3,DumpTree(t));
                            return null;
                        break;
                        case 3: // brackets
                            if (boolSymbolIndexing) {
                                strTemp =  this.GenerateStatement(cur,name,wBlock,nesting + 1);
                                if (strTemp == null) return null;
                                stmt += " (" + strTemp + ")";
                            } else {
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": ( unexpected",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            }
                        break;
                        case 4: // operator
                            // Put it without checking
                            strTemp =  this.GenerateStatement(cur,name,wBlock,nesting + 1);
                            if (strTemp == null) return null;
                            stmt += " (" + strTemp + ")";
                        break;
                        case 5: // keyword
                        case null:
                            // Put it without checking
                            strTemp =  this.GenerateStatement(cur,name,wBlock,nesting + 1);
                            if (strTemp == null) return null;
                            stmt += " (" + strTemp + ")";
                        break;
                        case 6: // literal
                            // Error
                            CR(0);
                            compiler.ReportError(cErr_SyntaxError,"In " + name + ": ( unexpected after literals",0);
                            compiler.ReportMessage(3,DumpTree(t));
                            return null;
                        break;
                    }
                    prev = 3;
                break;
                case "SquareBrackets":
                    // not allowed
                    CR(0);
                    compiler.ReportError(cErr_SyntaxError,"In " + name + ": Syntax error - square brackets found.",0);
                    compiler.ReportMessage(3,DumpTree(t));
                    return null;
                break;
                case "Keyword":
                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Keyword");
                    boolSymbolIndexing = false;
                    // If prev is dot - it is a property or method
                    if (prev == 2) {
                        // Word - symbol/part of a symbol
                        // Put it without additional checking
                        stmt += cur("__text");
                        prev = 1;
                    } else {
                        switch (cur("__text")) {
                            case "Then":
                            case "Else":
                                boolPossileSubCall = true;
                            break;
                            case "New":
                                if (!boolSetStatement) {
                                    CR(-1);
                                    compiler.ReportWarning(cErr_SyntaxError,"In " + name + ": Found New without Set statement.",0);
                                }
                                lastNew = i;
                                strClassInit += "." + project.contextName + " = ";
                            break;
                            case "Set":
                                if (i == 1 && nesting == 0) boolSetStatement = true;
                            break;
                            case "Option":
                                if (prev == null) {
                                    CR(-1);
                                    compiler.ReportWarning(cErr_SyntaxError,"Option statment ignored. Option Explicit is obsolete when using ASPC. Compiler will issue warnings for the undeclared variables.",0);
                                    return "";
                                } else {
                                    CR(0);
                                    compiler.ReportError(cErr_SyntaxError,"In " + name + ": Reserved keyword " + cur("__text") + " found.",0);
                                    compiler.ReportMessage(3,DumpTree(t));
                                    return null;
                                }
                            break;
                            default:
                                boolPossileSubCall = false;                        
                        }
                        // This is a keyword from the VBS language
                        stmt += " " + cur("__text");
                        prev = 5;                    
                    }
                break;
                case "Operator":
                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Operator");
                    boolSymbolIndexing = false;
                    // Depending on the operator type behavior should be different
                    if (cur("__text") == ".") {
                        // symbol continues
                        // previous must be brackets or word
                        if (prev != 1 && prev != 3) {
                            // This is an error if it is not a With block
                            if (wBlock != null) {
                                // *** Process With namespace ***
                                compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link (width)");
                                stmt += " " + this.LinkSymbol(wBlock,name);
                                stmt += ".";
                                prev = 2;
                            } else {
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": . unexpected",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            }
                        } else {
                            if (wBlock != null && (cur("SpaceBefore") == 1)) {
                                compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link (width)");
                                stmt += " " + this.LinkSymbol(wBlock,name);
                                stmt += ".";
                                prev = 2;
                            } else {
                                stmt += ".";
                                prev = 2;
                            }
                        }
                    } else {
                        if (cur("__text") == "=" && boolSetStatement) {
                            strClassInit = new String(stmt);
                        }
                        boolPossileSubCall = false;
                        switch (prev) {
                            case null:
                                if (nesting == 0) {
                                    compiler.ReportWarning(cErr_SyntaxError,"In " + name + ": operator found in the beginning of the statement",0);
                                }
                                stmt += " " + cur("__text");
                            break;
                            case 1: // word ok
                                // Put it without checking
                                stmt += " " + cur("__text");
                            break;
                            case 2: // dotmark - error
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": . unexpected",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            break;
                            case 3: // brackets
                                // Put it without checking
                                stmt += " " + cur("__text");
                            break;
                            case 4: // operator
                                // Put it without checking
                                stmt += " " + cur("__text");
                            break;
                            case 5: // keyword
                                // Put it without checking
                                stmt += " " + cur("__text");
                            break;
                            case 6: // literal
                                // Put it without checking
                                stmt += " " + cur("__text");
                            break;
                        }
                        prev = 4;
                    }
                break;
                case "Word":
                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Identifier");
                    boolSymbolIndexing = false;
                    // Word - symbol/part of a symbol
                    if (prev == null) {
                        // no previous item
                        compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link");
                        stmt += " " + this.LinkSymbol(cur("__text"),name);
                        prev = 1;
                    } else {
                        switch (prev) {
                            case 1: // word - error
                                if (boolPossileSubCall) {
                                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link");
                                    stmt += " " + this.LinkSymbol(cur("__text"),name);
                                    prev = 1;
                                    boolPossileSubCall = false;
                                    break;
                                }
                            case 6: // literal
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": Unexpected identifier (" + cur("__text") + ")",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            break;
                            case 2: // dotmark
                                // Put it without additional checking
                                stmt += cur("__text");
                                prev = 1;
                            break;
                            case 3: // brackets
                                // identifier after brackets is an error
                                CR(0);
                                compiler.ReportError(cErr_SyntaxError,"In " + name + ": Unexpected identifier (" + cur("__text") + ")",0);
                                compiler.ReportMessage(3,DumpTree(t));
                                return null;
                            break;
                            case 4: // operator
                                // It is the first part of indirection
                                compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link");
                                stmt += " " + this.LinkSymbol(cur("__text"),name);
                                prev = 1;
                            break;
                            case 5: // keyword
                                // It is the first part of indirection
                                
                                if (lastNew == (i-1)) {
                                    // Class initialization
                                    if (project.ClassExists(cur("__text")) != 0) {
                                        stmt += " " + cur("__text");
                                        if (this.parent.type == "Project") {
                                            // Unable to pass context
                                            CR(-1);
                                            compiler.ReportWarning(cERR_ClassnotFound,"In " + name + ": Unable to pass execution context to the created object in global function",0);
                                            compiler.ReportMessage(3,"LINE: " + DumpTree(t));
                                            strClassInit = "";
                                            return null;
                                        } else {
                                            if (this.parent.isRoot) {
                                                strClassInit += "Me";
                                            } else {
                                                strClassInit += project.contextName;
                                            }
                                        }
                                    } else {
                                        CR(0);
                                        compiler.ReportError(cERR_ClassnotFound,"In " + name + ": Class not defined in the project (" + cur("__text") + ")",0);
                                        compiler.ReportMessage(3,DumpTree(t));
                                        return null;
                                    }
                                } else {
                                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Link");
                                    stmt += " " + this.LinkSymbol(cur("__text"),name);
                                }
                                prev = 1;
                            break;
                        }
                    }
                break;
                case "TrailingComment":
                    compiler.ReportTrace(0,"Total statments processed: " + _g_processedStatementsCount + " " + i + " Trailing comment");
                    // End of statement
                    stmt += " " + cur("__text");
                    return stmt;
                break;
                default:
                    ///////////////
                    CR(0);
                    compiler.ReportError(cERR_UnExpected,"In " + name + ": Internal error - unexpected element (" + cur("__content") + ")",0);
                    compiler.ReportMessage(3,DumpTree(t));
                    return null;
            }
        }
    }
    if (nesting == 0 && lastNew >= 0 && !g_NoContext) return (stmt + "\r\n" + strClassInit);
    return stmt;
}


//////////////////////////////////////////// VBProject ////////////////////////////////////////

/*
  Class VBProject {
        VBProject( name );
        
        bool        CheckMembers();
        int         ClassExists(strClass,strSrcFile);
        
        bool        AddConstants(statement);
        bool        AddMethod(element);
        bool        AddLine(strKey,strValue);
        
        VBSClass    CreateRootClass(strClassName,strOutputFile,strInputFile);
        VBSClass    AddClass(strClass,strFile,strSrcFile);
    
        Boolean     ProcessFile(strProcType,strInputFile,strLoaderFile);
        Boolean     AnalyseFile(tree);
        Boolean     AnalyseClass(element);
        
        String      GenerateFile();
        Boolean     Generate();
        
    
  };


*/


//Class VBProject
    function VBProject(strName) {
        this.lines      = new Array();
        this.Name       = strName;
        this.name       = strName;
        this.classes    = new Array();
        
        this.symbols    = new Array();
        this.type       = "Project";
        this.access     = "Public";
        this.arguments  = new Array();
        this.body       = null;
        this.subscripts = null;
        this.isProperty = false;
        this.parent     = null;
        this.VBSymbols  = null;
        this.discarded  = new Array();
        
        this.contextName = cDefaultContextEntryName;
        this.mainProcName = cDefaultMainProcName;
        
        // Initialize some special things from the project
        var i; // Misc. counter
        var o;
        
        // Get the namespaces
        this.namespaces = new Array();
        o = cfg("VBSToVB")("EnvironmentObjects");
        for (i = 1; i <= o.Count; i++) {
            if (o(i) != 0) {
                this.namespaces[this.namespaces.length] = o.Key(i);
            }
        }
        // Load VBSymbols
        var strFile = cfg("VBSToVB")("VBSymbols");
        this.VBSymbols = CfgReader.Read(strFile);
    }
    VBProject.prototype.CheckMembers    = CheckMembers;
    VBProject.prototype.AddConstants    = AddConstants;
    VBProject.prototype.AddMethod       = AddMethod;
    VBProject.prototype.FindSymbol      = FindSymbol;
    VBProject.prototype.TraceDimStatement = TraceDimStatement;
    
    VBProject.prototype.CheckReserved   = function(s,name) {
        var i;
        for (i = 0; i < this.namespaces.length; i++) {
            if (s.toUpperCase() == this.namespaces[i].toUpperCase()) {
                CR(-1);
                compiler.ReportWarning(cERR_ReservedName,"In " + name + " namespace " + this.namespaces[i] + " is used. It is used by the DLL context and may cause unexpected behavior.",0);
            }
        }
    }
    VBProject.prototype.IsLinkable        = function(s) {
        var sym = this.FindSymbol(s);
        if (sym == null) {
            // Check in the VBSymbols
            if (typeof(this.VBSymbols(s)) == "object") {
                return true;
            }
            return false;
        } else {
            return true;
        }
    }
    VBProject.prototype.LinkSymbol      = function(s,name) {
        var sym = this.FindSymbol(s);
        if (sym == null) {
            // Check in the VBSymbols
            if (typeof(this.VBSymbols(s)) == "object") {
                if (this.VBSymbols(s)("Disable") == 1) {
                    CR(-1);
                    compiler.ReportWarning(cERR_UndeclaredSymbol,"In " + name + " symbol (" + s + ") is disabled in the symbols file. This may lead to unexpected results",0);
                    if ( typeof(this.VBSymbols(s)("Comment")) == "object") {
                        compiler.ReportMessage(2,this.VBSymbols(s)("Comment"));
                    }
                    return s;
                } else {
                    if (this.VBSymbols(s)("Replace") == 1) {
                        return this.VBSymbols(s)("Replacement");
                    }
                    return s;
                }
            }
            CR(-1);
            compiler.ReportWarning(cERR_UndeclaredSymbol,"In " + name + " symbol (" + s + ") is used but not declared. This may lead to unexpected results",0);
            return s;
        } else {
            return s;
        }
    }
    
    VBProject.prototype.CreateRootClass = function(strClassName,strOutputFile,strInputFile) {
        
        this.Root = this.AddClass(strClassName,strOutputFile,strInputFile);
        this.Root.isRoot = true;
        // Create symbols for the namespaces
        var i,o;
        for (i = 0; i < this.namespaces.length; i++) {
            o = new VBSymbol(this.Root);
            o.name = this.namespaces[i];
            o.type = "Variable";
            o.access = "Public";
            this.Root.AddSymbol(o);
        }
        o = new VBSymbol(this.Root);
        o.name = project.contextName;
        o.type = "Variable";
        o.access = "Private";
        return this.Root;
    }
    VBProject.prototype.ProcessFile = function(strProcType,strInputFile,strLoaderFile) {
                
        // Actual paths
        var strInputFilePath = CombinePaths(siteRootPath,strInputFile);
        //var strLoaderFilePath = CombinePaths(siteRootPathOutput,strLoaderFile);
        var strLoaderFilePath = strLoaderFile;
        
        // Report the file
        compiler.ReportFile(cFileInput,strInputFilePath,"Processing " + strInputFile + " (" + strProcType + ")");
        
        if (strProcType == "None") {
            return true;
        }
        
        var tmpFile = fso.CreateMemoryStream();
        var out;
    
        if (strProcType == "Copy") {
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnCreateOutput")) {
                    if (!procFileScript.script.OnCreateOutput(strLoaderFilePath)) {
                        compiler.ReportMessage(2,"CTS skipped file creation: " + strLoaderFilePath);
                        return true;
                    }
                }
            }
            if (!fso.FileExists(strInputFilePath)) {
                CR(0);
                compiler.ReportError(cERR_FileNotFound,"File not found: " + strInputFilePath,0);
                return false;
            }
            fso.CopyFile(strInputFilePath,strLoaderFilePath);
            compiler.ReportMessage(1,"File " + strInputFilePath + " copied to " + strLoaderFilePath);
            
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnOutputCreated")) {
                    out = fso.OpenFile(strLoaderFilePath,0x12);
                    procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                    out.Close();
                }
            }
            
            return true;
        }
        
        var tmpFileUsed = false;
        var tree;
        
        // Do SSI Parsing if requested and finish
        if (strProcType == "SSIToHTML") {
            SSIParser.Reload();
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnModifyParser")) {
                    procFileScript.script.OnModifyParser(SSIParser);
                }
            }
            tree = ConstructFile(strInputFilePath,SSIParser);
            if (!tree) return false;
            
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnCreateOutput")) {
                    if (!procFileScript.script.OnCreateOutput(strLoaderFilePath)) {
                        compiler.ReportMessage(2,"CTS skipped file creation: " + strLoaderFilePath);
                        return true;
                    }
                }
            }
            
            out = fso.CreateFile(strLoaderFilePath,0x1012);    
            tmpFileUsed = false;
            
            if (!CreateHTMLFileFromTree(tree,strInputFilePath,out)) {
                out.Close();
                return false;
            }
            compiler.ReportFile(cFileOutput,strLoaderFilePath,"Created HTML file");
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnOutputCreated")) {
                    out.Pos = 0;
                    procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                }
            }
            out.Close();            
            return true;
        }
        if (strProcType == "SSIWithASPTagsToHTML") {
            ASPParser.Reload();
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnModifyParser")) {
                    procFileScript.script.OnModifyParser(ASPParser);
                }
            }
            tree = ConstructFile(strInputFilePath,ASPParser);
            if (!tree) return false;
            
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnCreateOutput")) {
                    if (!procFileScript.script.OnCreateOutput(strLoaderFilePath)) {
                        compiler.ReportMessage(2,"CTS skipped file creation: " + strLoaderFilePath);
                        return true;
                    }
                }
            }
            
            out = fso.CreateFile(strLoaderFilePath,0x1012);    
            tmpFileUsed = false;
            
            if (!CreateHTMLFileFromTree(tree,strInputFilePath,out)) {
                out.Close();
                return false;
            }
            
            compiler.ReportFile(cFileOutput,strLoaderFilePath,"Created HTML file");
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnOutputCreated")) {
                    out.Pos = 0;
                    procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                }
            }
            out.Close();
            return true;
        }
        
        
        // Do ASP parsing if needed
        if (strProcType == "ASPtoVB" || strProcType == "ASPtoVBS" || strProcType == "ASPtoASPPlain" || strProcType == "ASPtoASP") {
            ASPParser.Reload();
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnModifyParser")) {
                    procFileScript.script.OnModifyParser(ASPParser);
                }
            }
            tree = ConstructFile(strInputFilePath,ASPParser);
            if (!tree) return false;
            
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnCreateOutput")) {
                    if (!procFileScript.script.OnCreateOutput(strLoaderFilePath)) {
                        compiler.ReportMessage(2,"CTS skipped file creation: " + strLoaderFilePath);
                        return true;
                    }
                }
            }
            
            if (strProcType == "ASPtoVBS") {
                out = fso.CreateFile(strLoaderFilePath,0x1012);
                if (!CreatePlainFileFromTree(tree,strInputFilePath,out)) {
                    out.Close();
                    return false;
                }
                
                if (procFileScript != null) {
                    if (procFileScript.IsDefined("OnOutputCreated")) {
                        out.Pos = 0;
                        procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                    }
                }
                out.Close();
                return true;
            }
            if (strProcType == "ASPtoASPPlain") {
                out = fso.CreateFile(strLoaderFilePath,0x1012);
                if (!CreateASPPlainFileFromTree(tree,strInputFilePath,out)) {
                    out.Close();
                    return false;
                }
                
                compiler.ReportFile(cFileOutput,strLoaderFilePath,"Created ASP file");
                if (procFileScript != null) {
                    if (procFileScript.IsDefined("OnOutputCreated")) {
                        out.Pos = 0;
                        procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                    }
                }
                out.Close();
                return true;
            }
            if (strProcType == "ASPtoASP") {
                out = fso.CreateFile(strLoaderFilePath,0x1012);
                if (!CreateASPFileFromTree(tree,strInputFilePath,out)) {
                    out.Close();
                    return false;
                }
                
                compiler.ReportFile(cFileOutput,strLoaderFilePath,"Created ASP file");
                if (procFileScript != null) {
                    if (procFileScript.IsDefined("OnOutputCreated")) {
                        out.Pos = 0;
                        procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                    }
                }
                out.Close();
                return true;
            }
            // ASPToVB
            //out = fso.CreateFile(tempFileName,0x1012);    
            out = tmpFile;
            tmpFileUsed = true;
            
            out.WriteText("'#! file " + strInputFilePath,1);
            if (!CreatePlainFileFromTree(tree,strInputFilePath,out)) {
                out.Close();
                return false;
            }
            out.Pos = 0;
            if (procFileScript != null) {
                if (procFileScript.IsDefined("OnOutputCreated")) {
                    out.Pos = 0;
                    procFileScript.script.OnOutputCreated(out,strLoaderFilePath);
                }
            }
            
        }
        
        // Do VBS processing if any
        
        if (strProcType == "ASPtoVB" || strProcType == "VBStoVB") {
            var     clsName;    // Generated class name for the root file
            var     clsFile;    // VB class file
            var     src;        // VBS src file
            var     tree;
            
            clsName = CreateClassModuleName(strLoaderFile);
            clsFile = CombinePaths(vbOutputPath,clsName + ".cls");
            
            // Create class for the page root
            this.CreateRootClass(clsName,clsFile,strInputFilePath);
            
            if (strProcType == "ASPtoVB") {
                //if (!fso.FileExists(tempFileName)) {
                //    CR(0);
                //    compiler.ReportError(cERR_FileNotFound,"Temp file not found: " + tempFileName,0);
                //    return false;
                //}
                src = tmpFile;
                src.Pos = 0;
            } else {
                if (!fso.FileExists(strInputFilePath)) {
                    CR(0);
                    compiler.ReportError(cERR_FileNotFound,"Input file not found: " + strInputFilePath,0);
                    return false;
                }
                src = fso.OpenFile(strInputFilePath,0x10);
            }
            
            // Trace the file using VBSParser
            var tree = VBSParser.ParseString(src.ReadText());
            src.Close();
             
            // We have the tree - collect the information
            this.AnalyseFile(tree);
            
            // End of the analysis
            
                if (strProcType == "ASPtoVB") {
                    this.CreateASPLoaderAndProperties(strLoaderFilePath,this.Root.name);
                }
                if (strProcType == "VBStoVB") {
                    this.CreateVBSLoaderAndProperties(strLoaderFilePath,this.Root.name);
                }
                
                if (procFileScript != null) {
                    if (procFileScript.IsDefined("OnLoaderFileCreated")) {
                        out = fso.OpenFile(strLoaderFilePath,0x12);
                        procFileScript.script.OnLoaderFileCreated(out,strLoaderFilePath);
                        out.Close();
                    }
                }
        }
        return true;
    }
    

    // Analyses syntax tree of a VBS file
    /* tree   -   input syntax tree
     *
     */
    VBProject.prototype.AnalyseFile = function(tree) {
        var el; // Element (node)
        var lastDirective = null;
        var mainProc = new ActiveXObject("newObjects.utilctls.VarDictionary");
        var i;
        var tempName = "s";
        var str;
        for (i = 1; i <= tree.Count; i++) {
            el = tree(i);
            // Process the different entries
            
            
            switch (el("__content")) {
                case "Dim":
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "DLLGlobal") {
                            this.TraceDimStatement(el,currentFile,true);
                            lastDirective = null;
                            break;
                        } else {
                            if (lastDirective("Type") == "DbgTrace") {
                                if (!g_isDbgBuild) {
                                    lastDirective = null;
                                    break;
                                }
                            } else {
                                compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Unexpected directive: " + lastDirective("Type") + " for a Dim statement",0);
                            }
                        }
                    } 
                    this.Root.TraceDimStatement(el,this.Root.srcFile);
                    lastDirective = null;
                break;
                case "Directive":
                    if (el("Type") == "file") {
                        lastDirective = null;
                        // set current file
                        currentFile = el("__text");
                        compiler.ReportFile(cFileHidden,currentFile,"Current file");
                    } else {
                        if (el("Type") == "ClassName") {
                            lastDirective = null;
                            str = new String(el("__text"));
                            str = Trim(str);
                            if (str.match(reClassName) != str) {
                                CR(-1)
                                compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Invalid class name in ClassName directive: " + el("_text"),0);
                            } else {
                                this.Root.name = str;
                            }
                        } else if (el("Type") == "AutoVariables") {
                            lastDirective = null;
                            this.Root.autovar = true;
                            compiler.ReportMessage(0,"AutoVariables enabled for " + this.Root.name);
                        } else {
                            // Preserve directive for the next statement
                            lastDirective = el;
                        }
                    }                        
                break;
                case "Function":
                case "Sub":
                case "Property":
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "DLLGlobal") {
                            if (el("__content") != "Property") {
                                this.AddMethod(el,true);
                                lastDirective = null;
                                break;
                            } else {
                                compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Unexpected directive: " + lastDirective("Type") + " properties cannot be declared DLLGlobal",0);
                            }
                        } else {
                            if (lastDirective("Type") == "DbgTrace") {
                                if (!g_isDbgBuild) {
                                    lastDirective = null;
                                    break;
                                }
                            } else {
                                compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Unexpected directive: " + lastDirective("Type") + " for " + el("FunctionName"),0);
                            }
                        }
                    }
                    this.Root.AddMethod(el);
                    lastDirective = null;
                break;
                case "Statement":
                case "With":
                case "Comment":
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "Discard") {
                            lastDirective = null;
                            this.discarded[this.discarded.length] = el;
                            break;
                        } else {
                            if (lastDirective("Type") == "DbgTrace") {
                                if (!g_isDbgBuild) {
                                    lastDirective = null;
                                    break;
                                }
                            } else {
                                compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Ignored directive: " + lastDirective("Type") + " for a regular statement",0);
                            }
                        }
                    }
                    mainProc.Add(tempName + i,el);
                    lastDirective = null;
                break;
                case "Const":
                    //*//
                    if (lastDirective != null) {
                        compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Ignored directive: " + lastDirective("Type") + " no directives are allowed for constants.",0);
                    }
                    if (el("Access") == "Private") {
                        this.Root.AddConstants(el,currentFile);
                    } else {
                        this.AddConstants(el,currentFile);
                    }
                    lastDirective = null;
                break;
                case "Class":
                    // Create class and pass the processing to its object
                    if (lastDirective != null) {
                        if (lastDirective("Type") == "Public") {
                            this.AnalyseClass(el,1);
                            lastDirective = null;
                            break;
                        } else {
                            if (lastDirective("Type") == "Creatable") {
                                this.AnalyseClass(el,2);
                                lastDirective = null;
                                break;
                            }
                        }
                        compiler.ReportWarning(cWarn_UnexpectedDirective,"In " + currentFile + " Ignored directive: " + lastDirective("Type") + " no directives are allowed for Class declarations.",0);
                    }
                    this.AnalyseClass(el,0);
                    lastDirective = null;
                break;
            }
        }
        // Add the main proc to the
        mainProc.Add("__content","Sub");
        mainProc.Add("Access","Public");
        mainProc.Add("Keyword","Sub");
        mainProc.Add("FunctionName",this.mainProcName);
        mainProc.Add("__text","()");
        mainProc.Add("__class","HTMLEmbedSearcher.newObjects.VBStmtSrch");
        this.Root.AddMethod(mainProc);
        // Check the members
        this.Root.CheckMembers("class " + this.Root.name);
        return true;
    }
    
    VBProject.prototype.AnalyseClass = function(el,pub) {
        var clsName; // class name
        var clsFile; // output file
        var o; // class Information 
        // Temporary variables
        var arr,tree;
        
         if (el("__class") == "HTMLEmbedSearcher.newObjects.VBStmtSrch") {
            switch (el("__content")) {
                case "Class":
                    clsName = el("ClassName");
                    clsFile = CombinePaths(vbOutputPath,clsName + ".cls");
                    if (project.ClassExists(clsName,currentFile) == 1) return true; // already defined
                    
                    if (project.ClassExists(clsName,currentFile) == -1) {
                        compiler.ReportError(5,clsName + ": Class with the same name is already defined in another file",0);
                        return false;
                    }
                    o = project.AddClass(clsName,clsFile,currentFile);
                    o.PublicType = pub;
                    return o.Analyse(el);
                break;
            }
        }                
        return true;
    }

    VBProject.prototype.AddClass = function(strClass,strFile,strSrcFile) {
        var o = new VBSClass(this,strClass,strFile,strSrcFile);
        
        // This class holds information about the class
        
        this.lines[this.lines.length] = {
            key:        "Class",
            value:      strClass + "; " + strFile,
            file:       strFile,
            srcFile:    strSrcFile,
            className:  strClass,
            info:       o
        };
        this.classes[this.classes.length] = o;
        // General data - should be imported from the config
        
        return o;
    }
    
    VBProject.prototype.AddLine = function(strKey,strValue) {
        this.lines[this.lines.length] = {
            key:        strKey,
            value:      strValue
        };
    }
    
    VBProject.prototype.CreateGlobalModule = function(ASPObjects) {
        var moduleName = this.Name + "_Globals";
        var moduleFile = moduleName + ".bas";
        compiler.ReportFile(cFileVB,CombinePaths(vbOutputPath,moduleFile),"Globals: .BAS module");
        this.AddLine("Module",moduleName + "; " + moduleFile);
        var glb = fso.CreateFile(CombinePaths(vbOutputPath,moduleFile),0x1012);
            glb.WriteText("Attribute VB_Name = \"" + moduleName + "\"",1);
            
            // Generate global symbols
            var i;
            for (i = 0; i < this.symbols.length; i++) {
                this.symbols[i].Generate(glb);
            }
        return true;
    }
    
    VBProject.prototype.ClassExists = function(strClass,strSrcFile) {
        var i;
        for (i = 0; i < this.lines.length; i++) {
            if (this.lines[i].key == "Class") {
                if (strClass == this.lines[i].className) {
                    if (strSrcFile == this.lines[i].srcFile) return 1; // exists but already defined
                    return -1;
                }
            }
        }
        return 0;
    }
    
    VBProject.prototype.GenerateFile = function() {
        
        var prj = fso.CreateFile(CombinePaths(vbOutputPath,this.Name + ".vbp"),0x1012);
        // Write standard records
        prj.WriteText("Type=OleDll",1);
        prj.WriteText("Startup=\"(None)\"",1);
        prj.WriteText("HelpFile=\"\"",1);
        prj.WriteText("ExeName32=\"" + this.Name + ".DLL\"",1);
        prj.WriteText("Name=\"" + classPrefix + "\"",1);
        prj.WriteText("CompilationType=0",1);
        prj.WriteText("OptimizationType=0",1);
        prj.WriteText("FavorPentiumPro(tm)=0",1);
        prj.WriteText("CodeViewDebugInfo=0",1);
        prj.WriteText("NoAliasing=0",1);
        prj.WriteText("BoundsCheck=0",1);
        prj.WriteText("OverflowCheck=0",1);
        prj.WriteText("FlPointCheck=0",1);
        prj.WriteText("FDIVCheck=0",1);
        prj.WriteText("UnroundedFP=0",1);
        prj.WriteText("StartMode=1",1);
        prj.WriteText("Unattended=-1",1);
        prj.WriteText("Retained=0",1);
        prj.WriteText("ThreadPerObject=0",1);
        prj.WriteText("MaxNumberOfThreads=1",1);
        
        
        // Write entries
        var i;
        for (i = 0; i < this.lines.length; i++) {
            prj.WriteText(this.lines[i].key + "=" + this.lines[i].value,1);
        }
        prj.Close();
        compiler.ReportFile(cFileVB,CombinePaths(vbOutputPath,this.Name + ".vbp"),"VB project file");
        return CombinePaths(vbOutputPath,this.Name + ".vbp");
    }
    VBProject.prototype.Generate = function() {
        // Generation
        var i;
        
        // Classes
        compiler.ReportMessage(0,"Generating classes");
        
        for (i = 0; i < this.classes.length; i++) {
            vbFiles ++;
            compiler.ReportMessage(1,"Class: " + this.classes[i].name);
            this.classes[i].Generate();
        }
        
        if (vbFiles > 0) {
            compiler.ReportMessage(0,"Generating global module");
            if (!this.CreateGlobalModule()) return false;
        } else {
            compiler.ReportMessage(0,"No VB files in the project");   
        }
    
        return true;
    }
    VBProject.prototype.AddDiscardedRec = function(node,file) {
        var i;
        var el;
        
        for (i = 1; i <= node.Count; i++) {
            el = node(i);
            switch (el("__content")) {
                case "Statement":
                    file.WriteText("' " + el("__text"),1);
                break;
                case "With":
                    file.WriteText("' With " + el("__text"),1);
                    this.AddDiscardedRec(el,file);
                    file.WriteText("' End With",1);
                break;
            }
        }
    }
    VBProject.prototype.AddDiscarded = function(file) {
        var i;
        var el;
        if (this.discarded.length > 0) {
            file.WriteText("' The following statements are discarded using the directive Discard",1);
            file.WriteText("' Usually it means you will use them to configure your component from outside - please do it.",1);
            file.WriteText("' ===========================================================================================",1);
        }
        for (i = 0; i < this.discarded.length; i++) {
            el = this.discarded[i];
            switch (el("__content")) {
                case "Statement":
                    file.WriteText("' " + el("__text"),1);
                break;
                case "With":
                    file.WriteText("' With " + el("__text"),1);
                    this.AddDiscardedRec(el,file);
                    file.WriteText("' End With",1);
                break;
            }
        }
    }
    
    VBProject.prototype.CreateASPLoaderAndProperties = function(strOutputASP,clsName) {
        var i;
        
        compiler.ReportFile(cFileOutput,strOutputASP,"ASP loader file");
        var asp = fso.CreateFile(strOutputASP,0x1012);
        // Generate common part of the ASP file
        asp.WriteText("<%",1);
        asp.WriteText("Dim proc",1);
        asp.WriteText("Set proc = Server.CreateObject(\"" + classPrefix + "." + clsName + "\")",1);
        
        for (i = 0; i < this.namespaces.length; i++) {
                // Add line in the ASP file
                asp.WriteText("Set proc." + this.namespaces[i] + " = " + this.namespaces[i],1);
        }
        this.AddDiscarded(asp);
        asp.WriteText("proc." + this.mainProcName,1);
        asp.WriteText("%>",1);
        asp.Close();
    }
    
    VBProject.prototype.CreateVBSLoaderAndProperties = function(strOutputASP,clsName) {
        var i;
        compiler.ReportFile(cFileOutput,strOutputASP,"VBS loader file");
        var asp = fso.CreateFile(strOutputASP,0x1012);
        // Generate common part of the ASP file
        asp.WriteText("Dim proc",1);
        asp.WriteText("Set proc = CreateObject(\"" + classPrefix + "." + clsName + "\")",1);
        
        for (i = 0; i < this.namespaces.length; i++) {
                // Add line in the ASP file
                asp.WriteText("Set proc." + this.namespaces[i] + " = " + this.namespaces[i],1);
        }
        this.AddDiscarded(asp);
        asp.WriteText("proc." + this.mainProcName,1);
        asp.Close();
    }





// Globals Initialization
//////////////////////////

var g_compilationResult = 1;

// cfg - contains project data

// Load parsers
compiler.ReportMessage(0,"Loading parser definitions from " + cfg("MAIN")("ParserDefinitions"));

// 1. ASP Parser
var ASPParser = new ActiveXObject("newObjects.utilctls.TextEmbedParser");

if (ASPParser.isDemoVersion) {
    compiler.ReportWarning(0,"DEMO WARNING! You are using a demo version - see the documentation for information about the limitations",0);
}

ASPParser.parserDefinitions = cfg("MAIN")("ParserDefinitions");
ASPParser.parserDefinition = cfg("ASPToPlain")("Parser");
compiler.ReportMessage(0,"Loaded ASP parser: " + ASPParser.parserDefinition);

// 2. VBS Parser
var VBSParser = new ActiveXObject("newObjects.utilctls.TextEmbedParser");
VBSParser.parserDefinitions = cfg("MAIN")("ParserDefinitions");
VBSParser.parserDefinition = cfg("VBStoVB")("Parser");
compiler.ReportMessage(0,"Loaded VBS parser:" + VBSParser.parserDefinition);

// 3. VBS Statment parser
var STMTParser = new ActiveXObject("newObjects.utilctls.TextEmbedParser");
STMTParser.parserDefinitions = cfg("MAIN")("ParserDefinitions");
if (typeof(cfg("VBStoVB")("StatementParser")) == "undefined") {
    STMTParser.parserDefinition = "STMT";
    compiler.ReportWarning(0,"Loading default statement parser definition.",0);
} else {
    STMTParser.parserDefinition = cfg("VBStoVB")("StatementParser");    
}
compiler.ReportMessage(0,"Loaded VBS statment parser:" + STMTParser.parserDefinition);

var SSIParser = new ActiveXObject("newObjects.utilctls.TextEmbedParser");
SSIParser.parserDefinitions = cfg("MAIN")("ParserDefinitions");
if (typeof(cfg("ASPToPlain")("SSIParser")) == "undefined") {
    SSIParser.parserDefinition = "SSI";
    compiler.ReportWarning(0,"Loading default SSI parser definition.",0);
} else {
    SSIParser.parserDefinition = cfg("ASPToPlain")("SSIParser");
}
compiler.ReportMessage(0,"Loaded SSI statment parser:" + SSIParser.parserDefinition);

  

// Determine misc global options
// 1. Temp file
var tempFileName = cfg("MAIN")("TempFile");
var siteRootPath = cfg("ASPToPlain")("SiteRoot");
var siteRootPathOutput = cfg("VBStoVB")("ASPOutputPath");
var vbOutputPath = cfg("VBStoVB")("VBOutputPath");
var fso = new ActiveXObject("newObjects.utilctls.SFMain");

var progIDNum = 0;

var g_projectName = cfg("MAIN")("ProjectName");
compiler.ReportMessage(0,"Determining project name: " + g_projectName);

var g_isDbgBuild = (cfg("MAIN")("DbgBuild") == 1);

var g_ResponseWrite = "Response.Write";
if (typeof(cfg("ASPToPlain")("Response.Write")) == "object") {
    if (cfg("ASPToPlain")("Response.Write") != "") {
        g_ResponseWrite = new String(cfg("ASPToPlain")("Response.Write"));
        compiler.ReportMessage(0,"Instead of Response.Write the following will be generated: " + g_ResponseWrite);
    }
}

var g_NoContext = false;
if (cfg("VBStoVB")("NoContext") == 1) g_NoContext = true;


var classPrefix;
classPrefix = g_projectName;
currentFile = "";

compiler.ReportMessage(1,"Class preffix: " + classPrefix);

compiler.ReportMessage(0,"Creating project manager");
var project = new VBProject(g_projectName);

compiler.ReportMessage(0,"Adding version information");
var n;
var verInfo = cfg("VERSION");
if (typeof(verInfo) == "object") {
    for (n = 1; n < verInfo.Count; n++) {
        project.AddLine(verInfo.Key(n),verInfo(n));
    }
}



if (!CreatePathsForFiles(siteRootPath,cfg("Files"))) {
    CR(0);
}

CreateDirectory(vbOutputPath);
compiler.ReportMessage(0,"Creating VB project directory: " + vbOutputPath);

var procFileName;
var procFileRelPath;
var procFileExt;
var procFileNameOnly;
var procFileType;
var procFileSection;
var procFileScript;

var vbFiles = 0;


function PrepareFileScript(sect) {
    var strScriptFile;
    var strScriptLanguage;
    var boolUseDefault = false;
    var info;
    if (sect("CompileTimeUseDefault") == 1) {
        // Use default
        if (typeof(cfg("CompileTimeScripting")("CompileTimeScript")) == "undefined") return null;
        strScriptFile = new String(cfg("CompileTimeScripting")("CompileTimeScript"));
        compiler.ReportMessage(0,"Loading default compile-time script:" + strScriptFile);        
        if (strScriptFile.length == 0) return null;
        if (typeof(cfg("CompileTimeScripting")("CompileTimeScriptLanguage")) == "undefined") {
            // Language not defined
            CR(0);
            compiler.ReportError(cErr_LanguageNotDef,"Default compile-time script language is not defined." ,0);        
            return null;
        }
        strScriptLanguage = new String(cfg("CompileTimeScripting")("CompileTimeScriptLanguage"));
        compiler.ReportMessage(0,"Default compile-time script language is:" + strScriptLanguage);
        boolUseDefault = true;
    } else {
        if (typeof(sect("CompileTimeScript")) == "undefined") return null;
        strScriptFile = new String(sect("CompileTimeScript"));
        compiler.ReportMessage(0,"Loading file compile-time script:" + strScriptFile);        
        if (strScriptFile.length == 0) return null;
        if (typeof(sect("CompileTimeScriptLanguage")) == "undefined") {
            // Language not defined
            CR(0);
            compiler.ReportError(cErr_LanguageNotDef,"File compile-time script language is not defined." ,0);        
            return null;
        }
        strScriptLanguage = new String(sect("CompileTimeScriptLanguage"));
        compiler.ReportMessage(0,"File compile-time script language is:" + strScriptLanguage);
    }
    // we have the file name
    var strScript = LoadScriptFromFile(strScriptFile,sect("File"));
    if (strScript == null) {
        CR(0);
        compiler.ReportError(cERR_FileNotFound,"Compile-time script not found: " + strScriptFile,0);        
        return null;
    }
    compiler.ReportMessage(0,"Loaded compile-time script:" + strScriptFile);    
    var scr = new ActiveXObject("newObjects.Scphost.ScpMan2");
    scr.Add("Project",cfg);
    scr.Add("Compiler",compiler);
    scr.Add("ProcessedFile",sect);
    
    var tempFileInfoSet; // Temp variable
    
    info = new ActiveXObject("newobjects.utilctls.VarDictionary");
    info.Add("CurrentFile",CombinePaths(siteRootPath,procFileName));
    info.Add("File",CombinePaths(siteRootPath,procFileName));
    info.Add("FileRelative",procFileName);
    info.Add("ProcType",procFileType);
    info.Add("SiteRootPath",siteRootPath);
    info.Add("SiteRootPathOutput",siteRootPathOutput);
    info.Add("FilePath",GetPathOf(CombinePaths(siteRootPath,procFileName)));
    info.Add("FileRelativePath",procFileRelPath);
    info.Add("FileNameOnly",procFileNameOnly);
    info.Add("FileExtension",procFileExt);
    
    tempFileInfoSet = GetCompileTimeScriptPath(strScriptFile,sect("File"));
    info.Add("ScriptFile",tempFileInfoSet);
    info.Add("ScriptFilePath",GetPathOf(tempFileInfoSet));
    info.Add("ScriptLanguage",strScriptLanguage);
    scr.Add("Info",info);
    
    // Find configuration
    // Empty config in case of none
    tempFileInfoSet = new ActiveXObject("newobjects.utilctls.VarDictionary");
    var strLibName = "";
    if (boolUseDefault) {
        if (typeof(cfg("CompileTimeScripting")) == "object") {
            if (typeof(cfg("CompileTimeScripting")("LibraryName")) == "object") {
                strLibName = cfg("CompileTimeScripting")("LibraryName");
                if (typeof(cfg("CompileTimeScripting")("CTSCONFIG")) == "object") {
                    if (typeof(cfg("CompileTimeScripting")("CTSCONFIG")(strLibName)) == "object") {
                        tempFileInfoSet = cfg("CompileTimeScripting")("CTSCONFIG")(strLibName);
                    }
                }    
            }
        }
    } else {
        if (typeof(sect("LibraryName")) == "object") {
            strLibName = sect("LibraryName");
            if (typeof(sect("CTSCONFIG")) == "object") {
                if (typeof(sect("CTSCONFIG")(strLibName)) == "object") {
                    tempFileInfoSet = sect("CTSCONFIG")(strLibName);
                }
            }    
        }
    }
    scr.Add("Configuration",tempFileInfoSet);
    
    if (typeof(cfg("VARIABLES")) == "object") {
        scr.Add("GlobalVariables",cfg("VARIABLES"));
    } else {
        tempFileInfoSet = new ActiveXObject("newobjects.utilctls.VarDictionary");
        scr.Add("GlobalVariables",tempFileInfoSet);
    }
    if (typeof(sect("VARIABLES")) == "object") {
        scr.Add("Variables",sect("VARIABLES"));
    } else {
        tempFileInfoSet = new ActiveXObject("newobjects.utilctls.VarDictionary");
        scr.Add("Variables",tempFileInfoSet);
    }
    
    // Load the engine
    if (!scr.LoadEngine(strScriptLanguage)) {
        CR(0);
        compiler.ReportError(cErr_EngineLoadFailed,"Failed to load specified script engine: " + strScriptLanguage,0);        
        scr.Clear();
        return null;
    }
    
    if (!scr.AddText(strScript)) {
        CR(0);
        compiler.ReportFile(cFileHidden,strScriptFile,"Error in this compile-time script file");
        compiler.ReportError(cErr_ScriptCompileErr,"Compile-time script error: " + scr.lastError,0);        
        scr.Clear();
        scr.Close();
        return null;
    }
    compiler.ReportMessage(0,"Compile-time script prepared");
    // Prepared
    // Run it
    if (!scr.Run()) {
        CR(0);
        compiler.ReportFile(cFileHidden,strScriptFile,"Error in this compile-time script file");
        compiler.ReportError(cErr_ScriptRunTimeErr,"Compile-time script run-time error: " + scr.lastError,0);        
        scr.Clear();
        scr.Close();
        return null;
    }
    compiler.ReportMessage(0,"Compile-time script initialized successfuly");
    return scr;
}

// Completion
var collCompletition = new ActiveXObject("newobjects.utilctls.VarDictionary");
collCompletition.enumItems = true;

function AskForCompletition() {
    var strFileName = procFileScript.script.OnRequireCompletion();
    if (strFileName == null || strFileName == "") return;
    
    compiler.ReportMessage(0,"Registering completion script: " + strFileName);
    
    var strFullName = procFileScript("Info")("ScriptFilePath") + "\\" + strFileName;
    if (!fso.FileExists(strFullName)) {
        compiler.ReportError(cERR_FileNotFound,"CTS requested non-existent completion script file.",0);
        CR(0);
        return;
    }
    // It exists
    
    var collComplFileInfo; // This will point the information for it
    var temp;
    // Check if already registered
    collComplFileInfo = collCompletition(strFullName);
    if (typeof(collComplFileInfo) != "object") {
        // Not registered yet
        collComplFileInfo = new ActiveXObject("newobjects.utilctls.VarDictionary");
        // Feed it up with the rewuired collections
        temp = new ActiveXObject("newobjects.utilctls.VarDictionary");
        temp.enumItems = true;
        temp.allowDuplicateNames = true;
        temp.allowUnnamedValues = true;
        collComplFileInfo.Add("Files",temp);
        // Deterine the script engine
        collComplFileInfo.Add("ScriptLanguage",procFileScript("Info")("ScriptLanguage"));
        // And add it
        collCompletition.Add(strFullName,collComplFileInfo);
    }
    
    var thisFile = new ActiveXObject("newobjects.utilctls.VarDictionary");
    // Add the data from this script
    thisFile.Add("ProcessedFile",procFileScript("ProcessedFile"));
    thisFile.Add("Info",procFileScript("Info"));
    thisFile.Add("Configuration",procFileScript("Configuration"));
    thisFile.Add("Variables",procFileScript("Variables"));
    
    collComplFileInfo("Files").Add(procFileName,thisFile);
}

function RunCompletitionScripts() {
    var i;
    var scr,file,cur,info;
    var strScrFile;
    
    for (i = 1; i <= collCompletition.Count; i++) {
        cur = collCompletition(i);
        strScrFile = collCompletition.Key(i);
        compiler.ReportMessage(0,"Executing completion script: " + strScrFile);
        if (!fso.FileExists(strScrFile)) {
            compiler.ReportError(cERR_FileNotFound,"CTS completion script file does not exist.",0);
            CR(0);
            return false;
        }
        scr = new ActiveXObject("newObjects.Scphost.ScpMan2");
        if (!scr.LoadEngine(cur("ScriptLanguage"))) {
            compiler.ReportError(cErr_EngineLoadFailed,"Invalid script language: " + cur("ScriptLanguage"),0);
            CR(0);
            return false;
        }
        // Feed the script with information
        scr.Add("Project",cfg);
        scr.Add("Compiler",compiler);
        scr.Add("GlobalVariables",cfg("VARIABLES"));
        
        // Prepare info
        info = new ActiveXObject("newobjects.utilctls.VarDictionary");
        info.Add("SiteRootPath",siteRootPath);
        info.Add("SiteRootPathOutput",siteRootPathOutput);
        info.Add("ScriptFile",strScrFile);
        info.Add("ScriptFilePath",GetPathOf(strScrFile));
        info.Add("ScriptLanguage",cur("ScriptLanguage"));

        scr.Add("Info",info);
        
        // And add the file collection
        scr.Add("Files",cur("Files"));
        
        file = fso.OpenFile(strScrFile,0x10);
        if (!scr.AddText(file.ReadText())) {
            CR(0);
            compiler.ReportFile(cFileHidden,strScrFile,"Error in this compile-time script file");
            compiler.ReportError(cErr_ScriptCompileErr,"Compile-time script error: " + scr.lastError,0);        
            scr.Clear();
            scr.Close();
            return false;
        }
        // Ready to run
        if (!scr.Run()) {
            CR(0);
            compiler.ReportFile(cFileHidden,strScrFile,"Error in this compile-time script file");
            compiler.ReportError(cErr_ScriptRunTimeErr,"Compile-time script run-time error: " + scr.lastError,0);        
            scr.Clear();
            scr.Close();
            return false;
        }
        // Completed
        scr.Stop();
        scr.Clear();
        scr.Close();
    }
    return true;
}

function Compile(nOptions) {
    if (g_compilationResult == 0) {
        compiler.ReportResult(g_compilationResult,"");
        return true;
    }
    // Enum process
    var enmFiles = cfg("Files");
    
    var strASPOutputFile;
    var tempFileInfoSet;
    var tempOutputRelative;
    var tmpStr,tmpArr;
    
    //var g_globals = project.CreateGlobalModule(cfg("VBStoVB")("EnvironmentObjects"));
    for (n = 1; n <= enmFiles.Count; n++) {
        if (enmFiles(n)("Exclude") == 1) continue;
        procFileName = GetRelPath(siteRootPath,enmFiles(n)("File"));
        procFileRelPath = GetPathOf(procFileName);
        compiler.ReportMessage(0,"File rel name: " + procFileName);
        compiler.ReportMessage(0,"File relative path: " + procFileRelPath);
        tmpArr = procFileName.split("\\");
        if (typeof(tmpArr) != "object") {
            compiler.ReportError(cERR_FileNotFound,"Wrong file in the project: " + enmFiles(n)("File"),0);
            compiler.ReportResult(0,"");
            return;
        }
        tmpStr = tmpArr[tmpArr.length - 1];
        tmpArr = tmpStr.split(".");
        if (typeof(tmpArr) != "object") {
            compiler.ReportWarning(cERR_FileNotFound,"File has no extension: " + enmFiles(n)("File"),0);
            procFileExt = "";    
            procFileNameOnly = tmpStr;
        } else {
            procFileExt = tmpArr[tmpArr.length - 1];    
            procFileNameOnly = tmpStr.slice(0,tmpStr.length - procFileExt.length - 1);
        }
        compiler.ReportMessage(0,"File name only: " + procFileNameOnly);
        compiler.ReportMessage(0,"File extension: " + procFileExt);
        

        procFileType = enmFiles(n)("Process");
        procFileSection = enmFiles(n);            
                
        if (procFileName.length > 0) {
        
            compiler.ReportFile(cFileHidden,CombinePaths(siteRootPath,procFileName),"Main Input file");
        
            procFileScript = PrepareFileScript(procFileSection);
            
            if (procFileScript == null) {
                // Check if it is not an error
                if (g_compilationResult == 0) {
                    // An error occured - skip this file processing
                    continue;
                }
                // No script file - default processing will be performed
                strASPOutputFile = CombinePaths(siteRootPathOutput,procFileName);
                if (!project.ProcessFile(procFileType,procFileName,strASPOutputFile)) {
                    CR(0);
                }
            } else {
                // There is a script here
                // Call OnProcessFile
                var boolProcessOnce = true;
                while (true) {
                    if (procFileScript.IsDefined("OnProcessFile")) {
                        tempOutputRelative = procFileScript.script.OnProcessFile(procFileName);
                        boolProcessOnce = false;
                    } else {
                        tempOutputRelative = procFileName;
                        boolProcessOnce = true;
                    }
                    // If the return result is not empty process the file
                    if (tempOutputRelative == null || tempOutputRelative == "") {
                        // No more processing 
                        break;
                    }
                    strASPOutputFile = CombinePaths(siteRootPathOutput,tempOutputRelative);
                    CreateDirectory(GetPathOf(strASPOutputFile));
                    if (!project.ProcessFile(procFileType,procFileName,strASPOutputFile)) {
                        CR(0);
                        break;
                    }
                    if (boolProcessOnce) break;
                }
                // Ask the script for completition requirements
                if (g_compilationResult) {
                    if (procFileScript.IsDefined("OnRequireCompletion")) {
                        compiler.ReportMessage(0,"Asking for completion requirements");
                        AskForCompletition();
                    } 
                }
                procFileScript.Stop();
                procFileScript.Close();
                procFileScript.Clear();
            }
        }
    }
    
    if (g_compilationResult) {
        compiler.ReportMessage(0,"Running completion scripts");
        if (!RunCompletitionScripts()) {
            compiler.ReportError(cErr_MiscErr,"Completion completed with errors.",0);
            CR(0);
        }
    }
    
    var prjFile = "";
    
    project.Generate();
    
    if (vbFiles > 0) {
        if (g_compilationResult) {
            prjFile = project.GenerateFile();
        }
    } else {
        prjFile = "";
    }
    compiler.ReportResult(g_compilationResult,prjFile);
}



