' Namespaces
' ~~~~~~~~~~
' Project                 - Project tree
' Compiler                - Compiler object
' ProcessedFile           - File sction from the project
' Info                    - A collection of information about the file
' Configuration           - Configuration section or empty collection if no config is supplied
' GlobalVariables         - Configuration section or empty collection if no config is supplied



' Initialization Begin
' ~~~~~~~~~~~~~~~~~~~~

Dim query, dbFile, PageSize ' Variables read from the configuration
Dim bDefaultSettings
bDefaultSettings = False

' READ THE CONFIGURATION
' This is not required, but in this example we will assume some default
' settings to make it easier to test it.
If Configuration("QUERY") = "" Or Configuration("DBFile") = "" Then bDefaultSettings = True

If bDefaultSettings Then
    query = "SELECT * FROM Items"
    ' All the paths supplied to the CTS are without ending "\"
    dbFile = Info("SiteRootPath") & "\db\" & "db.mdb"
Else
    query = Configuration("QUERY")
    dbFile = Configuration("DBFile")
End If    
If Configuration("PageSize") = "" Then
    PageSize = 5
Else
    PageSize = CInt(Configuration("PageSize"))
End If


Dim numFile, bFail
numFile = 0
bFail = False

Dim db, rst ' DB variables

' Now prepare the DB
Set db = CreateObject("ADODB.Connection")
    Compiler.ReportMessage 4,"Open DB: " & dbFile
    On Error Resume Next
    db.Open "Driver={Microsoft Access Driver (*.mdb)};DBQ=" & dbFile & ";"
    If Err.Number <> 0 Then
        Compiler.ReportError -1,Err.Description,0
        bFail = True
    End If

Set rst = CreateObject("ADODB.Recordset")
    Compiler.ReportMessage 4,"Executing query: " & query
    rst.Open query,db,1,3,1
    If Err.Number <> 0 Then
        Compiler.ReportError -1,Err.Description,0
        bFail = True
    End If
On Error Goto 0

' Initialization End
' ~~~~~~~~~~~~~~~~~~

' Events
' ~~~~~~
Function OnProcessFile(fileVirtName)
    numFile = numFile + 1
    If bFail Then
        OnProcessFile = ""
        Exit Function
    End If
    If Not rst.EOF Then
        OnProcessFile = Info("FileRelativePath") & "\" & Info("FileNameOnly") & numFile & "." & Info("FileExtension")
    Else
        On Error Resume Next
        rst.Close
        db.Close
        Set rst = Nothing
        Set db = Nothing
        OnProcessFile = ""
    End If
End Function

Function OnCheckStructure(Tree,Scripts,Customs)
    ' This is for demonstration only
    ' Compiler.ReportMessage 4,"Custom tags count = " & Customs.Count
    
    ' Chek if it exists. 
    ' By default Customs collection is configured to return empty value for the missing elements.
    ' (See VarDictionary object for further information on the VarDictionary universal collection.)
    If Not IsObject(Customs("LIST")) Then
        Compiler.ReportError -1,"LIST custom tag not found",0
        OnCheckStructure = False
        Exit Function
    End If
    
    Dim template,node ' We will clone new node every time a new row is added        
    Dim i,n ' Misc counters
        
    ' Get a copy of the original node. We will use it as a template
    'Compiler.ReportMessage 4,"Clone the original node"
    Set template = Customs("LIST").Clone
    
    ' Clear the original node
    Customs("LIST").Clear
    
    ' And make it a container
    Customs("LIST")("Class") = "Container"
    ' Now we have an empty container and a copy in the template variable
        
    ' And cycle through the results of the query from the current record further
    ' Compiler.ReportMessage 4,"Begin cycle"
    n = 0
    While Not rst.EOF And n < PageSize
    
        ' clone the template to prepare new row for the list
        Set node = template.Clone
        
        ' Make it a container
        node("Class") = "Container"
        
        ' Cycle through the template elements
        'Compiler.ReportMessage 4,"Subelements=" & node.Count
        For i = 1 To node.Count
        
            ' We are interested only by the objects. The values
            ' of the CUSTOM tag represent it HTML tag Attributes and can be very useful.
            ' But this example doesn't use them
            If IsObject(node(i)) Then
                
                ' We want to replace all the <%=X%> nodes with the corresponding fields from the DB
                If node(i)("Class") = "Assignment" Then
                    
                    'Compiler.ReportMessage 4,"  " & node.Key(i) & "  " & node(i)("Class") & " " & node(i)("Content")
                    
                    ' Transforming them to normal text nodes
                    node(i)("Class") = "Text"
                    
                    ' And replacing their content with the field from the recordset
                    ' We are using the fact that original content is equal to the field name
                    ' See the item.vbs for another way to do this. Which is more convenient
                    ' depends on how complex the CTS will be and what is more important - 
                    ' easy-to-write template html or simplier CTS.
                    On Error Resume Next
                    node(i)("Content") = rst(node(i)("Content")).Value
                    If Err.Number <> 0 Then
                        ' If the field does not exist or something else is wrong
                        Compiler.ReportError -3, Err.Description,0
                    End If
                    On Error Goto 0
                End If
            End If
        Next
        
        ' Now we need to add this container to the original node (LIST)
        ' We are using an empty name. Some descriptive name may help if additional
        ' processing will be done by a custom procedure later, but this is
        ' rarely needed. For example in this case some rows can be named to indicate
        ' something special about them and a code written below may want to modify them...
        i = Customs("LIST").Add("",node)
        
        ' Let report we've done
        ' Compiler.ReportMessage 4,"Generated Page #" & numFile & " Row #" & n
        rst.MoveNext
        n = n + 1
    Wend
    
    ' Now the prev/next links remain
    If IsObject(Customs("PREV")) And IsObject(Customs("NEXT")) Then
        ' We require the both tags to exist. 
        
        If numFile > 1 Then
            Customs("PREV")("Class") = "Text"
            Customs("PREV")("Content") = "<A HREF=""" & Info("FileNameOnly") & numFile-1 & "." & Info("FileExtension") & """>Previous page</A>"
        End If        
        If Not rst.EOF Then
            Customs("NEXT")("Class") = "Text"
            Customs("NEXT")("Content") = "<A HREF=""" & Info("FileNameOnly") & numFile+1 & "." & Info("FileExtension") & """>Next page</A>"    
        End If
    End If
        
    ' That's all the element is modified and the original
    ' Custom node became Container with subcontainer for every row
    OnCheckStructure = True
End Function

' No need of the other event in this case
Sub OnOutputCreated(file,strFile)
    ' TO DO:
    '   Inspect the output file if needed
End Sub

Sub OnLoaderFileCreated(file,strFile)
    ' TO DO:
    '   Inspect the output loader file if needed
End Sub