' Namespaces
' ~~~~~~~~~~
' Project                 - Project tree
' Compiler                - Compiler object
' ProcessedFile           - File sction from the project
' Info                    - A collection of information about the file
' Configuration           - Configuration section or empty collection if no config is supplied
' GlobalVariables         - Configuration section or empty collection if no config is supplied


' Initialization Begin
' ~~~~~~~~~~~~~~~~~~~~

Dim query, dbFile ' Variables read from the configuration
Dim bDefaultSettings
bDefaultSettings = False

' READ THE CONFIGURATION
' This is not required, but in this example we will assume some default
' settings to make it easier to test it.
If Configuration("QUERY") = "" Or Configuration("DBFile") = "" Then bDefaultSettings = True

If bDefaultSettings Then
    query = "SELECT * FROM Items"
    ' All the paths supplied to the CTS are without ending "\"
    dbFile = Info("SiteRootPath") & "\db\" & "db.mdb"
Else
    query = Configuration("QUERY")
    dbFile = Configuration("DBFile")
End If    

Dim numFile
numFile = 0
Dim db, rst ' DB variables
Dim bFail
bFail = False

' Prepare the DB
' This happens while the script is loaded before calling any function
Set db = CreateObject("ADODB.Connection")
    Compiler.ReportMessage 4,"Open DB: " & dbFile
    On Error Resume Next
    db.Open "Driver={Microsoft Access Driver (*.mdb)};DBQ=" & dbFile & ";"
    If Err.Number <> 0 Then
        Compiler.ReportError -1,Err.Description,0
        bFail = True
    End If
    
Set rst = CreateObject("ADODB.Recordset")
    Compiler.ReportMessage 4,"Executing query: " & query
    rst.Open query,db,1,3,1
    If Err.Number <> 0 Then
        Compiler.ReportError -1,Err.Description,0
        bFail = True
    End If
    On Error Goto 0
    
' Initialization End
' ~~~~~~~~~~~~~~~~~~

' Events
' ~~~~~~
Function OnProcessFile(fileVirtName)
    numFile = numFile + 1
    If bFail Then
        OnProcessFile = ""
        Exit Function
    End If
    If Not rst.EOF Then
        ' Generate the name of the new file
        OnProcessFile = Info("FileRelativePath") & "\" & Info("FileNameOnly") & numFile & "." & Info("FileExtension")
    Else
        On Error Resume Next
        rst.Close
        db.Close
        Set rst = Nothing
        Set db = Nothing
        OnProcessFile = ""
    End If
End Function

Function OnCheckStructure(Tree,Scripts,Customs)
   
    Dim node ' temp variable
    Dim i ' Misc counter
        
    
    For i = 1 To Customs.Count
        ' If the CUSTOM tag contains FIELD attribute place a field in it
        If VarType(Customs(i)("FIELD")) = vbString Then 
            ' Compiler.ReportMessage 4,"Adding Field:" & Customs(i)("FIELD")
            
            ' Perform the corresponding operation to the tag type
            Select Case Customs(i)("TYPE")
            Case "IMAGE"
                On Error Resume Next
                    If rst(Customs(i)("FIELD")).Value <> "" Then
                        Customs(i)("Class") = "Text"
                        Customs(i)("Content") = "<IMG SRC=""" & rst(Customs(i)("FIELD")).Value & """>"
                    Else
                        Customs(i)("Class") = "Text"
                        Customs(i)("Content") = "<B>No Image</B>"
                    End If
                    If Err.Number <> 0 Then
                        Compiler.ReportWarning -2,Err.Description,0
                    End If
                On Error Goto 0
            Case Else
                ' If the field does not exist we should deal with the error
                On Error Resume Next
                Customs(i)("Content") = rst(Customs(i)("FIELD")).Value
                If Err.Number <> 0 Then
                    Compiler.ReportWarning -2,Err.Description,0
                Else
                    ' To prevent missing fields to appear we are changing their type to
                    ' Text only if no error occurs
                    Customs(i)("Class") = "Text"
                End If
                On Error Goto 0
            End Select
        End If
    Next
        
    ' Position the recordset one step ahead
    rst.MoveNext
    
    ' Chek if the PREV and NEXT exist
    If IsObject(Customs("PREV")) And IsObject(Customs("NEXT")) Then
        ' The PREV and NEXT should not have FIELD attribute and will
        ' not be processed by the cycle above
        
        ' And place them
        If numFile > 1 Then
            Customs("PREV")("Class") = "Text"
            Customs("PREV")("Content") = "<A HREF=""" & Info("FileNameOnly") & numFile-1 & "." & Info("FileExtension") & """>Previous page</A>"
        End If        
        If Not rst.EOF Then
            Customs("NEXT")("Class") = "Text"
            Customs("NEXT")("Content") = "<A HREF=""" & Info("FileNameOnly") & numFile+1 & "." & Info("FileExtension") & """>Next page</A>"    
        End If    
    End If
    
        
    OnCheckStructure = True
End Function

Sub OnOutputCreated(file,strFile)
    ' TO DO:
    '   Inspect the output file if needed
End Sub

Sub OnLoaderFileCreated(file,strFile)
    ' TO DO:
    '   Inspect the output loader file if needed
End Sub