Function MapFilePath(str)
    Dim r
    
    If InStr(1,xmlFile,"://") > 0 Then
        ' URL
        MapFilePath = str
        Exit Function
    End If
    
    r = Replace(str,"/","\")
    
    If Left(r,2) = "\\" Then
        ' Network path
        MapFilePath = r
    ElseIf Left(r,1) = "\" Then
        ' Virtual
        MapFilePath = Info("SiteRootPath") & "\" & r
    Else
        ' Relative
        MapFilePath = Info("FilePath") & "\" & r
    End If
End Function

Sub Transform(xmlSource,boolXMLIsFile,xslTemplate,boolXSLIsFile)
    Dim xmlFile, src, xsl, b
      Set src = CreateObject("Msxml2.DOMDocument")
      Set xsl = CreateObject("Msxml2.DOMDocument")
      src.async = False
      xsl.async = False
      
      On Error Resume Next
        If boolXMLIsFile Then
            b = src.load(xmlSource)
        Else
            b = src.loadXML(xmlSource)
        End If
                
        If Err.Number <> 0 Then
            Compiler.ReportError -1,Err.Description,0
            Exit Sub
        ElseIf Not b Then
            If boolXMLIsFile Then
                Compiler.ReportError -1,"Cannot load: " & xmlFile,0
            Else
                Compiler.ReportError -1,"Cannot load the XML source",0
            End If
            Exit Sub
        End If
           
        If boolXSLIsFile Then
            b = xsl.load(xslTemplate)
        Else
            b = xsl.loadXML(xslTemplate)
        End If
        
        If Err.Number <> 0 Then
            Compiler.ReportError -2,Err.Description,0
            Exit Sub
        ElseIf Not b Then
            If boolXSLIsFile Then
                Compiler.ReportError -2,"Failed to load the content of the tag as template.",0
            Else
                Compiler.ReportError -2,"Cannot load the XSL template",0
            End If
            Exit Sub            
        End If
        
        node("Class") = "Text"
        node("Content") = ""
        node("Content") = src.TransformNode(xsl)
        If Err.Number <> 0 Then
            Compiler.ReportError -3,Err.Description,0
            Exit Sub
        End If
End Sub

Sub Proceed
    Dim xml,xsl,bxmlFile,bxslFile
    
    If node("TEMPLATE") <> "" Then
        ' file
        bxslFile = True
        xsl = MapFilePath(node("TEMPLATE"))
        Compiler.ReportMessage 4,"Using template from file: " & xsl
    ElseIf node("TEMPLATENODE") <> "" Then
        ' Other node
        Compiler.ReportMessage 4,"Using template from node: " & node("TEMPLATENODE")
        If Not IsObject(Tree(node("TEMPLATENODE"))) Then
            Compiler.ReportError -2,"Node (" & node("TEMPLATENODE") & ") does not exist",0
            Exit Sub
        End If
        bxslFile = False
        xsl = Tree(node("TEMPLATENODE"))("Content")
    Else
        ' This node content
        bxslFile = False
        xsl = node("Content")
        Compiler.ReportMessage 4,"Using template from the tag content"
    End If
    
    If node("FILE") <> "" Then
        ' File
        bxmlFile = True
        xml = MapFilePath(node("FILE"))
        Compiler.ReportMessage 4,"Loading source from file: " & xml
    ElseIf node("SRCELEMENT") <> "" Then
        ' Other node
        Compiler.ReportMessage 4,"Loading source from node: " & node("SRCELEMENT")
        If Not IsObject(Tree(node("SRCELEMENT"))) Then
            Compiler.ReportError -2,"Node (" & node("SRCELEMENT") & ") does not exist",0
            Exit Sub
        End If
        bxmlFile = False
        xml = Tree(node("SRCELEMENT"))("Content")
    Else
        ' This node content
        bxmlFile = False
        xml = node("Content")
        Compiler.ReportMessage 4,"Using template from the tag content"
    End If
    
    Transform xml,bxmlFile,xsl,bxslFile
End Sub

Proceed    
       
       
       